/***************************************************************************/
/* 		This code is part of WWW grabber called pavuk		   */
/*		Copyright (c) 1997 - 2000 Stefan Ondrejicka		   */
/*		(ondrej@idata.sk)					   */
/*		Distributed under GPL 2 or later			   */
/***************************************************************************/

#include "config.h"
#include "gui.h"

#ifdef GTK_FACE

#include <gdk/gdkkeysyms.h>
#include <string.h>

#include "uconfig.h"
#include "lfname.h"
#include "lang.h"
#include "charset.h"
#include "form.h"

#include "icons/append.xpm"
#include "icons/modify.xpm"
#include "icons/clear.xpm"
#include "icons/delete.xpm"
#include "icons/browse.xpm"
#include "icons/ok.xpm"
#include "icons/apply.xpm"
#include "icons/limit.xpm"
#include "icons/cancel.xpm"

extern char *get_strategie_label(int);

static void CfgCommon(object , func_data)
GtkObject *object;
gpointer func_data;
{
	if (!xget_cfg_values_comm())
	{
		if (cfg.use_prefs)
			cfg_dump_pref();
	}
}

static void LangListInsertList(object , func_data)
GtkObject *object;
gpointer func_data;
{
	listanfo *lnfo = (listanfo *)func_data;
	GList *dlist = GTK_CLIST(lnfo->entry)->selection;
	char *p;
	char pom[3] = {'\0','\0','\0'};

	if (!dlist) gdk_beep();

        while(dlist)
        {
		gtk_clist_get_text(GTK_CLIST(lnfo->entry) ,
			GPOINTER_TO_INT(dlist->data), 0 , &p); 

		strncpy(pom , p , 2);
		p = pom;
		gtk_clist_append(GTK_CLIST(lnfo->list) , &p);
                dlist = dlist->next;
        }

}

/*** URL ***/

struct cfg_url_dlg_t {
	GtkWidget	*url_list;
	GtkWidget	*url_entry;
	GtkWidget	*ext_sw;
	GtkWidget	*ext_frame;
	GtkWidget	*method;
	GtkWidget	*encoding;
	GtkWidget	*field_list;
	GtkWidget	*field_type;
	GtkWidget	*field_name;
	GtkWidget	*field_value_l;
	GtkWidget	*field_value_t;
	GtkWidget	*field_value_f;
	GtkWidget	*tab;
};

static url_info *cfg_url_dlg_get_url_info(urldlg)
struct cfg_url_dlg_t *urldlg;
{
	url_info *ui;
	char *p;
	int row;

	p = gtk_entry_get_text(GTK_ENTRY(urldlg->url_entry));

	ui = url_info_new(p);
	ui->type = URLI_FORM;

	if (urldlg->method)
		ui->method = GTK_OPTION_MENU(urldlg->method)->menu_item ?
			(int) gtk_object_get_user_data(GTK_OBJECT(GTK_OPTION_MENU(urldlg->method)->menu_item)) : FORM_M_GET;

	if (urldlg->encoding)
		ui->encoding = GTK_OPTION_MENU(urldlg->encoding)->menu_item ?
			(int) gtk_object_get_user_data(GTK_OBJECT(GTK_OPTION_MENU(urldlg->encoding)->menu_item)) : FORM_E_URLENCODED;

	for (row = 0 ; row < GTK_CLIST(urldlg->field_list)->rows ; row++)
	{
		form_field *ff;

		ff = _malloc(sizeof(form_field));

		ff->type = (int) gtk_clist_get_row_data(GTK_CLIST(urldlg->field_list), row);

		gtk_clist_get_text(GTK_CLIST(urldlg->field_list) , row , 1 , &p);
		ff->name = form_decode_urlencoded_str(p , strlen(p));

		gtk_clist_get_text(GTK_CLIST(urldlg->field_list) , row , 2 , &p);
		ff->value = form_decode_urlencoded_str(p , strlen(p));

		ui->fields = dllist_append(ui->fields, ff);
	}

	return ui;
}

static void cfg_url_dlg_append(w, urldlg)
GtkWidget *w;
struct cfg_url_dlg_t *urldlg;
{
	char *p,*s;
	bool ok = TRUE;
	int row;

	p = gtk_entry_get_text(GTK_ENTRY(urldlg->url_entry));

	if (p && *p)
	{
		url_info *ui = NULL;
		protocol pt = URLT_UNKNOWN;

		s = url_parse_scheme(p);

		if (s)
		{
			pt = url_scheme_to_schemeid(s);
			ok = prottable[pt].supported;
			_free(s);
		}
		else
			pt = URLT_HTTP;

		if (!ok)
		{
			/* unsupported URL type */
			gdk_beep();
			return;
		}

		if ((urldlg->ext_sw && GTK_TOGGLE_BUTTON(urldlg->ext_sw)->active) ||
			!urldlg->ext_sw)
		{
			if (pt != URLT_HTTP && pt != URLT_HTTPS)
			{
				/* extended request supported only with HTTP */
				gdk_beep();
				return;
			}
			ui = cfg_url_dlg_get_url_info(urldlg);
		}

		gtk_clist_freeze(GTK_CLIST(urldlg->url_list));
		row = gtk_clist_append(GTK_CLIST(urldlg->url_list) , &p);

		if (ui)
			gtk_clist_set_row_data_full(GTK_CLIST(urldlg->url_list), row, ui, (GtkDestroyNotify)url_info_free);
		gtk_clist_thaw(GTK_CLIST(urldlg->url_list));
	}
	else gdk_beep();
}

static void cfg_url_dlg_modify(w, urldlg)
GtkWidget *w;
struct cfg_url_dlg_t *urldlg;
{
	char *p,*s;
	bool ok = TRUE;
	int row;

	if (!GTK_CLIST(urldlg->url_list)->selection)
	{
		gdk_beep();
		return;
	}

	row = GPOINTER_TO_INT(GTK_CLIST(urldlg->url_list)->selection->data);

	p = gtk_entry_get_text(GTK_ENTRY(urldlg->url_entry));

	if (p && *p)
	{
		url_info *ui;
		protocol pt = URLT_UNKNOWN;

		s = url_parse_scheme(p);

		if (s)
		{
			pt = url_scheme_to_schemeid(s);
			ok = prottable[pt].supported;
			_free(s);
		}
		else
			pt = URLT_HTTP;

		if (!ok)
		{
			/* unsupported URL type !!! */
		}

		ui = (url_info *) gtk_clist_get_row_data(GTK_CLIST(urldlg->url_list), row);
		if (ui)
			url_info_free(ui);
		ui = NULL;

		if ((urldlg->ext_sw && GTK_TOGGLE_BUTTON(urldlg->ext_sw)->active) ||
			!urldlg->ext_sw)
		{
			if (pt != URLT_HTTP && pt != URLT_HTTPS)
			{
				/* extended request supported only with HTTP */
				gdk_beep();
				return;
			}
			ui = cfg_url_dlg_get_url_info(urldlg);
		}

		gtk_clist_set_text(GTK_CLIST(urldlg->url_list), row, 0, p);

		if (ui)
			gtk_clist_set_row_data_full(GTK_CLIST(urldlg->url_list), row, ui, (GtkDestroyNotify)url_info_free);
	}
	else gdk_beep();
}

static void cfg_url_dlg_switch(w, row, col, event, urldlg)
GtkObject *w;
int row;
int col;
GdkEvent *event;
struct cfg_url_dlg_t *urldlg;
{
	if (GTK_CLIST(w)->selection)
	{
		url_info *ui;
		char *p;

		ui = (url_info *) gtk_clist_get_row_data(GTK_CLIST(urldlg->url_list), row);

		if (urldlg->ext_sw)
		{
			gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(urldlg->ext_sw) , ui != NULL);
			gtk_widget_set_sensitive(urldlg->ext_frame, ui != NULL);
		}

		gtk_clist_get_text(GTK_CLIST(urldlg->url_list) , row , 0 , &p); 

		gtk_entry_set_text(GTK_ENTRY(urldlg->url_entry), p);

		if (ui)
		{
			dllist *ptr;

			if (urldlg->method)
				gtk_option_menu_set_history(GTK_OPTION_MENU(urldlg->method), ui->method);
			if (urldlg->encoding)
				gtk_option_menu_set_history(GTK_OPTION_MENU(urldlg->encoding), ui->encoding);

			gtk_clist_clear(GTK_CLIST(urldlg->field_list));

			ptr = ui->fields;
			while (ptr)
			{
				char *p3[3];
				int row;
				form_field *ff = (form_field *) ptr->data;

				switch(ff->type)
				{
					case FORM_T_FILE:
						p3[0] = "FILE";
					break;
					case FORM_T_TEXTAREA:
						p3[0] = "LONG TEXT";
					break;
					default:
						p3[0] = "TEXT";
				}

				p3[1] = form_encode_urlencoded_str(ff->name);
				p3[2] = form_encode_urlencoded_str(ff->value);

				row = gtk_clist_append(GTK_CLIST(urldlg->field_list), p3);
				gtk_clist_set_row_data(GTK_CLIST(urldlg->field_list),
					row, (gpointer)ff->type);

				_free(p3[1]);
				_free(p3[2]);
				ptr = ptr->next;
			}
		}
	}
}
	
static void cfg_url_dlg_field_append(w, urldlg)
GtkWidget *w;
struct cfg_url_dlg_t *urldlg;
{
	char *p3[3];
	int type;

	type = GTK_OPTION_MENU(urldlg->field_type)->menu_item ?
		(int) gtk_object_get_user_data(GTK_OBJECT(GTK_OPTION_MENU(urldlg->field_type)->menu_item)) : 0;

	switch(type)
	{
		case FORM_T_FILE:
			p3[0] = "FILE";
			p3[2] = form_encode_urlencoded_str(
				gtk_entry_get_text(GTK_ENTRY(urldlg->field_value_f)));
		break;
		case FORM_T_TEXTAREA:
		{
			int len;
			char *p;

			p3[0] = "LONG TEXT";
			len = gtk_text_get_length(GTK_TEXT(urldlg->field_value_t));
			p = gtk_editable_get_chars(GTK_EDITABLE(urldlg->field_value_t), 0, len);
			p3[2] = form_encode_urlencoded_str(p);
			g_free(p);
		}
		break;
		default:
			p3[0] = "TEXT";
			p3[2] = form_encode_urlencoded_str(
				gtk_entry_get_text(GTK_ENTRY(urldlg->field_value_l)));
	}

	p3[1] = form_encode_urlencoded_str(gtk_entry_get_text(GTK_ENTRY(urldlg->field_name)));

	if (p3[0] && p3[1] && p3[2] && *p3[0] && *p3[1] && *p3[2])
	{
		int row = gtk_clist_append(GTK_CLIST(urldlg->field_list), p3);
		gtk_clist_set_row_data(GTK_CLIST(urldlg->field_list),
			row, (gpointer)type);
	}
	else
		gdk_beep();

	_free(p3[2]);
	_free(p3[1]);
}

static void cfg_url_dlg_field_modify(w, urldlg)
GtkWidget *w;
struct cfg_url_dlg_t *urldlg;
{
	char *p3[3];
	int type;
	int row;

	if (!GTK_CLIST(urldlg->field_list)->selection)
	{
		gdk_beep();
		return;
	}

	row = GPOINTER_TO_INT(GTK_CLIST(urldlg->field_list)->selection->data);

	type = GTK_OPTION_MENU(urldlg->field_type)->menu_item ?
		(int) gtk_object_get_user_data(GTK_OBJECT(GTK_OPTION_MENU(urldlg->field_type)->menu_item)) : 0;

	switch(type)
	{
		case FORM_T_FILE:
			p3[0] = "FILE";
			p3[2] = form_encode_urlencoded_str(
				gtk_entry_get_text(GTK_ENTRY(urldlg->field_value_f)));
		break;
		case FORM_T_TEXTAREA:
		{
			int len;
			char *p;

			p3[0] = "LONG TEXT";
			len = gtk_text_get_length(GTK_TEXT(urldlg->field_value_t));
			p = gtk_editable_get_chars(GTK_EDITABLE(urldlg->field_value_t), 0, len);
			p3[2] = form_encode_urlencoded_str(p);
			g_free(p);
		}
		break;
		default:
			p3[0] = "TEXT";
			p3[2] = form_encode_urlencoded_str(
				gtk_entry_get_text(GTK_ENTRY(urldlg->field_value_l)));
	}

	p3[1] = form_encode_urlencoded_str(gtk_entry_get_text(GTK_ENTRY(urldlg->field_name)));

	if (p3[0] && p3[1] && p3[2] && *p3[0] && *p3[1] && *p3[2])
	{
		gtk_clist_set_text(GTK_CLIST(urldlg->field_list),
			row, 0 , p3[0]);
		gtk_clist_set_text(GTK_CLIST(urldlg->field_list),
			row, 1 , p3[1]);
		gtk_clist_set_text(GTK_CLIST(urldlg->field_list),
			row, 2 , p3[2]);
		gtk_clist_set_row_data(GTK_CLIST(urldlg->field_list),
			row, (gpointer)type);
	}
	else
		gdk_beep();

	_free(p3[2]);
	_free(p3[1]);
}

static void cfg_url_dlg_set_type(w, urldlg)
GtkWidget *w;
struct cfg_url_dlg_t *urldlg;
{
	int type;
	int tab;

	type = (int) gtk_object_get_user_data(GTK_OBJECT(w));

	switch (type)
	{
		case FORM_T_FILE:
			tab = 1;
		break;
		case FORM_T_TEXTAREA:
			tab = 2;
		break;
		default:
			tab = 0;
	}

	gtk_notebook_set_page(GTK_NOTEBOOK(urldlg->tab), tab);
}

static void cfg_url_dlg_field_switch(w, row, col, event, urldlg)
GtkObject *w;
int row;
int col;
GdkEvent *event;
struct cfg_url_dlg_t *urldlg;
{
	if (GTK_CLIST(w)->selection)
	{
		char *p;
		int type;

		type = (int) gtk_clist_get_row_data(GTK_CLIST(urldlg->field_list), row);

		gtk_clist_get_text(GTK_CLIST(urldlg->field_list) , row , 1 , &p); 
		p = form_decode_urlencoded_str(p, strlen(p));
		gtk_entry_set_text(GTK_ENTRY(urldlg->field_name), p);
		_free(p);

		gtk_clist_get_text(GTK_CLIST(urldlg->field_list) , row , 2 , &p); 
		p = form_decode_urlencoded_str(p, strlen(p));

		switch (type)
		{
			case FORM_T_FILE:
				gtk_option_menu_set_history(GTK_OPTION_MENU(urldlg->field_type) , 1);
				gtk_entry_set_text(GTK_ENTRY(urldlg->field_value_f) , p);
				gtk_notebook_set_page(GTK_NOTEBOOK(urldlg->tab), 1);
			break;
			case FORM_T_TEXTAREA:
				gtk_option_menu_set_history(GTK_OPTION_MENU(urldlg->field_type) , 2);
				gtk_text_set_point(GTK_TEXT(urldlg->field_value_t), 0);
				gtk_text_forward_delete(GTK_TEXT(urldlg->field_value_t),
					gtk_text_get_length(GTK_TEXT(urldlg->field_value_t)));
				gtk_text_insert(GTK_TEXT(urldlg->field_value_t), NULL, NULL, NULL, p, strlen(p));
				gtk_notebook_set_page(GTK_NOTEBOOK(urldlg->tab), 2);
			break;
			default:
				gtk_option_menu_set_history(GTK_OPTION_MENU(urldlg->field_type) , 0);
				gtk_entry_set_text(GTK_ENTRY(urldlg->field_value_l) , p);
				gtk_notebook_set_page(GTK_NOTEBOOK(urldlg->tab), 0);
		}
		_free(p);
	}
}

static void cfg_url_dlg_set_sensitive(w, urldlg)
GtkWidget *w;
struct cfg_url_dlg_t *urldlg;
{
	gtk_widget_set_sensitive(urldlg->ext_frame,
		GTK_TOGGLE_BUTTON(urldlg->ext_sw)->active);
}

static void cfg_url_dlg_chose_file_ok(w, fdata)
GtkWidget *w;
gpointer fdata;
{
	char *p;
	GtkWidget *chf;

	chf = gtk_widget_get_toplevel(w);

	p = gtk_file_selection_get_filename(GTK_FILE_SELECTION(chf));

	gtk_entry_set_text(GTK_ENTRY(fdata), p);
}

static void cfg_url_dlg_chose_file(w, fdata)
GtkWidget *w;
gpointer fdata;
{
	static GtkWidget *chf = NULL;

	if (!chf)
	{
		chf = gtk_file_selection_new(gettext("Pavuk: Choose form field file"));
		gtk_signal_connect(GTK_OBJECT(chf), "destroy",
			GTK_SIGNAL_FUNC(gtk_widget_destroyed), &chf);

		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(chf)->ok_button),
			"clicked", GTK_SIGNAL_FUNC(cfg_url_dlg_chose_file_ok), fdata);

		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(chf)->ok_button),
			"clicked", GTK_SIGNAL_FUNC(guitl_PopdownW), chf);

		gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(chf)->cancel_button),
			"clicked", GTK_SIGNAL_FUNC(guitl_PopdownW), chf);
	}

	gtk_widget_show(chf);
	if (GTK_WIDGET_REALIZED(chf))
		gdk_window_raise(chf->window);
}

static void cfgtab_url(notebook)
GtkWidget *notebook;
{
	GtkWidget *label,*box,*tbox,*ptab,*prow,*menu,*mi;
	GtkWidget *swin,*brow,*button,*pbox;
	GtkWidget *hsb,*vsb;
	GtkAdjustment *hadj,*vadj;
	static listanfo p;
	static struct cfg_url_dlg_t urldlg;

	tbox = gtk_hbox_new(FALSE, 2);
	gtk_widget_show(tbox);
	label = gtk_label_new (gettext("URL"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), tbox , label);

	box = gtk_table_new(3 , 1 , FALSE);
	gtk_box_pack_start(GTK_BOX(tbox), box, TRUE, TRUE, 1);
	gtk_widget_show(box);

	swin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_table_attach_defaults(GTK_TABLE(box), swin , 0 , 1, 0, 1);
	gtk_widget_show(swin);

	gui_cfg.url_list = gtk_clist_new(1);
	urldlg.url_list = gui_cfg.url_list;
	gtk_clist_set_selection_mode(GTK_CLIST(gui_cfg.url_list), GTK_SELECTION_BROWSE);
#ifdef _GTK_FEATURES_1_2
	gtk_clist_set_reorderable(GTK_CLIST(gui_cfg.url_list) , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(gui_cfg.url_list) , 0 , TRUE);
#endif
	gtk_container_add(GTK_CONTAINER(swin), gui_cfg.url_list);
	gtk_widget_show(gui_cfg.url_list);

	brow = gtk_table_new(1 , 2 , FALSE);
	gtk_table_attach(GTK_TABLE(box), brow , 0, 1, 1, 2,
			GTK_EXPAND | GTK_FILL, GTK_FILL, 2 , 5);
	gtk_widget_show(brow);

	gui_cfg.url_entry = guitl_tab_add_entry(brow , gettext("Request URL: "), 0 , 0 , FALSE);
	urldlg.url_entry = gui_cfg.url_entry;

	gtk_signal_connect(GTK_OBJECT(gui_cfg.url_list) , "select_row" , 
			GTK_SIGNAL_FUNC(cfg_url_dlg_switch) , 
			(gpointer) &urldlg);

	brow = gtk_hbutton_box_new();
	gtk_table_attach(GTK_TABLE(box), brow , 0, 1, 2, 3,
			GTK_EXPAND | GTK_FILL, GTK_FILL, 2 , 5);
	gtk_hbutton_box_set_spacing_default(1);
	gtk_widget_show(brow);

	gtk_button_box_set_layout(GTK_BUTTON_BOX(brow) , GTK_BUTTONBOX_SPREAD);

	button = guitl_pixmap_button(append_xpm , NULL ,gettext("Append"));
	gtk_container_border_width(GTK_CONTAINER(button), 0);
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_widget_show(button);

	p.list = gui_cfg.url_list;
	p.entry = gui_cfg.url_entry;

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
		GTK_SIGNAL_FUNC(cfg_url_dlg_append) , 
		(gpointer) &urldlg);

	gtk_signal_connect(GTK_OBJECT(gui_cfg.url_entry) , "activate" ,
		GTK_SIGNAL_FUNC(cfg_url_dlg_append) , 
		(gpointer) &urldlg);

	button = guitl_pixmap_button(modify_xpm , NULL , gettext("Modify"));
	gtk_container_border_width(GTK_CONTAINER(button), 0);
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
		GTK_SIGNAL_FUNC(cfg_url_dlg_modify) , 
		(gpointer) &urldlg);

	button = guitl_pixmap_button(clear_xpm , NULL , gettext("Clear"));
	gtk_container_border_width(GTK_CONTAINER(button), 0);
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
		GTK_SIGNAL_FUNC(guitl_ListClear) , 
		(gpointer) gui_cfg.url_list);

	button = guitl_pixmap_button(delete_xpm , NULL , gettext("Delete"));
	gtk_container_border_width(GTK_CONTAINER(button), 0);
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
		GTK_SIGNAL_FUNC(guitl_ListDeleteSelected) , 
		(gpointer) gui_cfg.url_list);

#ifdef _GTK_FEATURES_1_2
	/*** DRAG'N'DROP ***/
  	gtk_drag_dest_set(box , GTK_DEST_DEFAULT_ALL,
			dragtypes , NUM_ELEM(dragtypes) - 1 ,
               		GDK_ACTION_COPY | GDK_ACTION_MOVE);
  	gtk_signal_connect(GTK_OBJECT(box) ,
			"drag_data_received" ,
			GTK_SIGNAL_FUNC(gui_window_drop_url),
			NULL);
#endif

	box = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(tbox), box, TRUE, TRUE, 1);
	gtk_widget_show(box);

	urldlg.ext_sw = gtk_check_button_new_with_label(gettext("Extended informations for HTTP POST request"));
	gtk_box_pack_start(GTK_BOX(box), urldlg.ext_sw, FALSE, FALSE, 1);
	gtk_widget_show(urldlg.ext_sw);

	gtk_signal_connect(GTK_OBJECT(urldlg.ext_sw), "toggled" ,
		GTK_SIGNAL_FUNC(cfg_url_dlg_set_sensitive) , &urldlg);

	urldlg.ext_frame = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(box), urldlg.ext_frame, FALSE, FALSE, 1);
	gtk_widget_show(urldlg.ext_frame);

	ptab = gtk_vbox_new(FALSE,5);
	gtk_container_add(GTK_CONTAINER(urldlg.ext_frame), ptab);
	gtk_widget_show(ptab);

	prow = gtk_table_new(2, 2, FALSE);
	gtk_box_pack_start(GTK_BOX(ptab), prow, FALSE, FALSE, 1);
	gtk_widget_show(prow);

	label = gtk_label_new(gettext("Request method: "));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0 , 0.5);
	gtk_table_attach(GTK_TABLE(prow), label, 0, 1, 0, 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	gtk_widget_show(label);

	urldlg.method = gtk_option_menu_new();
	gtk_table_attach(GTK_TABLE(prow), urldlg.method, 1, 2, 0, 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	menu = gtk_menu_new();
	mi = gtk_menu_item_new_with_label("GET");
	gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)FORM_M_GET);
	gtk_menu_append(GTK_MENU(menu), mi);
	gtk_widget_show(mi);
	mi = gtk_menu_item_new_with_label("POST");
	gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)FORM_M_POST);
	gtk_menu_append(GTK_MENU(menu), mi);
	gtk_widget_show(mi);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(urldlg.method), menu);
	gtk_widget_show(urldlg.method);

	label = gtk_label_new(gettext("Request encoding: "));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0 , 0.5);
	gtk_table_attach(GTK_TABLE(prow), label, 0, 1, 1, 2, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	gtk_widget_show(label);

	urldlg.encoding = gtk_option_menu_new();
	gtk_table_attach(GTK_TABLE(prow), urldlg.encoding, 1, 2, 1, 2, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	menu = gtk_menu_new();
	mi = gtk_menu_item_new_with_label("multipart/form-data");
	gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)FORM_E_MULTIPART);
	gtk_menu_append(GTK_MENU(menu), mi);
	gtk_widget_show(mi);
	mi = gtk_menu_item_new_with_label("application/x-www-form-urlencoded");
	gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)FORM_E_URLENCODED);
	gtk_menu_append(GTK_MENU(menu), mi);
	gtk_widget_show(mi);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(urldlg.encoding), menu);
	gtk_widget_show(urldlg.encoding);

	label = gtk_label_new(gettext("Query fields: "));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0 , 0.5);
	gtk_box_pack_start(GTK_BOX(ptab), label, FALSE, FALSE, 2);
	gtk_widget_show(label);

	swin = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_usize(swin , -1, 100);
	gtk_box_pack_start(GTK_BOX(ptab), swin, TRUE, TRUE, 2);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_widget_show(swin);

	urldlg.field_list = gtk_clist_new(3);
	gtk_clist_set_selection_mode(GTK_CLIST(urldlg.field_list), GTK_SELECTION_BROWSE);
	gtk_clist_set_column_title(GTK_CLIST(urldlg.field_list), 0 , gettext("Type"));
	gtk_clist_set_column_title(GTK_CLIST(urldlg.field_list), 1 , gettext("Name"));
	gtk_clist_set_column_title(GTK_CLIST(urldlg.field_list), 2 , gettext("Value"));
	gtk_clist_column_titles_show(GTK_CLIST(urldlg.field_list));
#ifdef _GTK_FEATURES_1_2
	gtk_clist_set_column_auto_resize(GTK_CLIST(urldlg.field_list) , 0 , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(urldlg.field_list) , 1 , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(urldlg.field_list) , 2 , TRUE);
#endif
	gtk_container_add(GTK_CONTAINER(swin), urldlg.field_list);
	gtk_widget_show(urldlg.field_list);

	gtk_signal_connect(GTK_OBJECT(urldlg.field_list) , "select_row" , 
			GTK_SIGNAL_FUNC(cfg_url_dlg_field_switch) , 
			(gpointer) &urldlg);

	prow = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(ptab), prow , FALSE, FALSE, 1);
	gtk_widget_show(prow);

	button = guitl_pixmap_button(append_xpm , NULL ,gettext("Append"));
	gtk_container_add(GTK_CONTAINER(prow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button), "clicked" ,
		GTK_SIGNAL_FUNC(cfg_url_dlg_field_append), &urldlg);

	button = guitl_pixmap_button(modify_xpm , NULL ,gettext("Modify"));
	gtk_container_add(GTK_CONTAINER(prow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button), "clicked" ,
		GTK_SIGNAL_FUNC(cfg_url_dlg_field_modify), &urldlg);

	button = guitl_pixmap_button(delete_xpm , NULL ,gettext("Delete"));
	gtk_container_add(GTK_CONTAINER(prow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
		GTK_SIGNAL_FUNC(guitl_ListDeleteSelected) , 
		(gpointer) urldlg.field_list);

	prow = gtk_table_new(2,3, FALSE);
	gtk_box_pack_start(GTK_BOX(ptab), prow, FALSE, FALSE, 1);
	gtk_widget_show(prow);

	label = gtk_label_new(gettext("Type: "));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0 , 0.5);
	gtk_table_attach(GTK_TABLE(prow), label, 0, 1, 0, 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	gtk_widget_show(label);

	urldlg.field_type = gtk_option_menu_new();
	gtk_table_attach(GTK_TABLE(prow), urldlg.field_type, 1, 2, 0, 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	menu = gtk_menu_new();
	mi = gtk_menu_item_new_with_label("TEXT");
	gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)FORM_T_TEXT);
	gtk_menu_append(GTK_MENU(menu), mi);
	gtk_widget_show(mi);
	gtk_signal_connect(GTK_OBJECT(mi) , "activate" , 
		GTK_SIGNAL_FUNC(cfg_url_dlg_set_type) , (gpointer) &urldlg);
	mi = gtk_menu_item_new_with_label("FILE");
	gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)FORM_T_FILE);
	gtk_menu_append(GTK_MENU(menu), mi);
	gtk_widget_show(mi);
	gtk_signal_connect(GTK_OBJECT(mi) , "activate" , 
		GTK_SIGNAL_FUNC(cfg_url_dlg_set_type) , (gpointer) &urldlg);
	mi = gtk_menu_item_new_with_label("LONG TEXT");
	gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)FORM_T_TEXTAREA);
	gtk_menu_append(GTK_MENU(menu), mi);
	gtk_widget_show(mi);
	gtk_signal_connect(GTK_OBJECT(mi) , "activate" , 
		GTK_SIGNAL_FUNC(cfg_url_dlg_set_type) , (gpointer) &urldlg);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(urldlg.field_type), menu);
	gtk_widget_show(urldlg.field_type);

	urldlg.field_name = guitl_tab_add_entry(prow , gettext("Name: "), 0 , 1 , FALSE);

	label = gtk_label_new(gettext("Value: "));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0 , 0.0);
	gtk_table_attach(GTK_TABLE(prow), label, 0, 1, 2, 3, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	gtk_widget_show(label);

	urldlg.tab = gtk_notebook_new();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(urldlg.tab), FALSE);
	gtk_table_attach(GTK_TABLE(prow), urldlg.tab, 1, 2, 2, 3, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	gtk_widget_show(urldlg.tab);

	label = gtk_label_new("TEXT");
	brow = gtk_vbox_new(FALSE, 1);
	gtk_widget_show(brow);
	gtk_notebook_append_page(GTK_NOTEBOOK(urldlg.tab), brow , label);

	urldlg.field_value_l = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(brow), urldlg.field_value_l , FALSE, FALSE , 1);
	gtk_widget_show(urldlg.field_value_l);

	label = gtk_label_new("FILE");
	brow = gtk_vbox_new(FALSE, 1);
	gtk_widget_show(brow);
	gtk_notebook_append_page(GTK_NOTEBOOK(urldlg.tab), brow , label);

	pbox = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(brow), pbox , FALSE, FALSE, 1);
	gtk_widget_show(pbox);

	urldlg.field_value_f = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(pbox), urldlg.field_value_f , TRUE, TRUE , 1);
	gtk_widget_show(urldlg.field_value_f);

	button = guitl_pixmap_button(browse_xpm , NULL ,gettext("Browse ..."));
	gtk_box_pack_start(GTK_BOX(pbox), button , FALSE, FALSE , 1);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button), "clicked" ,
		GTK_SIGNAL_FUNC(cfg_url_dlg_chose_file), urldlg.field_value_f);

	label = gtk_label_new("LONG TEXT");
	brow = gtk_table_new(2,2,FALSE);
	gtk_widget_show(brow);
	gtk_notebook_append_page(GTK_NOTEBOOK(urldlg.tab), brow , label);

	hadj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0));
	hsb = gtk_hscrollbar_new(hadj);
	gtk_table_attach(GTK_TABLE(brow), hsb, 0, 1, 1, 2,
		GTK_EXPAND | GTK_FILL, GTK_FILL, 0 , 0);
	gtk_widget_show(hsb);
 
	vadj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0));
	vsb = gtk_vscrollbar_new(vadj);
	gtk_table_attach(GTK_TABLE(brow), vsb, 1, 2, 0, 1,
		GTK_FILL, GTK_EXPAND | GTK_FILL, 0 , 0);
	gtk_widget_show(vsb);

	urldlg.field_value_t = gtk_text_new(hadj, vadj);
	gtk_widget_set_usize(urldlg.field_value_t, -1, 100);
	gtk_text_set_editable(GTK_TEXT(urldlg.field_value_t), TRUE);
	gtk_table_attach(GTK_TABLE(brow), urldlg.field_value_t, 0, 1, 0, 1,
		GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0 , 0);
	gtk_widget_show(urldlg.field_value_t);

	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(urldlg.ext_sw), FALSE);
	gtk_widget_set_sensitive(urldlg.ext_frame, FALSE);
}


/* FORM DATA */
static void cfgtab_formdata(notebook)
GtkWidget *notebook;
{
	GtkWidget *label,*box,*tbox,*ptab,*prow,*menu,*mi;
	GtkWidget *swin,*brow,*button,*pbox;
	GtkWidget *hsb,*vsb;
	GtkAdjustment *hadj,*vadj;
	static listanfo p;
	static struct cfg_url_dlg_t urldlg;

	urldlg.ext_sw = NULL;
	urldlg.encoding = NULL;
	urldlg.method = NULL;

	tbox = gtk_hbox_new(FALSE, 2);
	gtk_widget_show(tbox);
	label = gtk_label_new (gettext("Form data"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), tbox , label);

	box = gtk_table_new(3 , 1 , FALSE);
	gtk_box_pack_start(GTK_BOX(tbox), box, TRUE, TRUE, 1);
	gtk_widget_show(box);

	swin = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_table_attach_defaults(GTK_TABLE(box), swin , 0 , 1, 0, 1);
	gtk_widget_show(swin);

	gui_cfg.formdata_list = gtk_clist_new(1);
	urldlg.url_list = gui_cfg.formdata_list;
	gtk_clist_set_selection_mode(GTK_CLIST(gui_cfg.formdata_list), GTK_SELECTION_BROWSE);
#ifdef _GTK_FEATURES_1_2
	gtk_clist_set_reorderable(GTK_CLIST(gui_cfg.formdata_list) , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(gui_cfg.formdata_list) , 0 , TRUE);
#endif
	gtk_container_add(GTK_CONTAINER(swin), gui_cfg.formdata_list);
	gtk_widget_show(gui_cfg.formdata_list);

	brow = gtk_table_new(1 , 2 , FALSE);
	gtk_table_attach(GTK_TABLE(box), brow , 0, 1, 1, 2,
			GTK_EXPAND | GTK_FILL, GTK_FILL, 2 , 5);
	gtk_widget_show(brow);

	gui_cfg.formdata_entry = guitl_tab_add_entry(brow , gettext("Matching action URL: "), 0 , 0 , FALSE);
	urldlg.url_entry = gui_cfg.formdata_entry;

	gtk_signal_connect(GTK_OBJECT(gui_cfg.formdata_list) , "select_row" , 
			GTK_SIGNAL_FUNC(cfg_url_dlg_switch) , 
			(gpointer) &urldlg);

	brow = gtk_hbutton_box_new();
	gtk_table_attach(GTK_TABLE(box), brow , 0, 1, 2, 3,
			GTK_EXPAND | GTK_FILL, GTK_FILL, 2 , 5);
	gtk_hbutton_box_set_spacing_default(1);
	gtk_widget_show(brow);

	gtk_button_box_set_layout(GTK_BUTTON_BOX(brow) , GTK_BUTTONBOX_SPREAD);

	button = guitl_pixmap_button(append_xpm , NULL ,gettext("Append"));
	gtk_container_border_width(GTK_CONTAINER(button), 0);
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_widget_show(button);

	p.list = gui_cfg.formdata_list;
	p.entry = gui_cfg.formdata_entry;

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
		GTK_SIGNAL_FUNC(cfg_url_dlg_append) , 
		(gpointer) &urldlg);

	gtk_signal_connect(GTK_OBJECT(gui_cfg.formdata_entry) , "activate" ,
		GTK_SIGNAL_FUNC(cfg_url_dlg_append) , 
		(gpointer) &urldlg);

	button = guitl_pixmap_button(modify_xpm , NULL , gettext("Modify"));
	gtk_container_border_width(GTK_CONTAINER(button), 0);
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
		GTK_SIGNAL_FUNC(cfg_url_dlg_modify) , 
		(gpointer) &urldlg);

	button = guitl_pixmap_button(clear_xpm , NULL , gettext("Clear"));
	gtk_container_border_width(GTK_CONTAINER(button), 0);
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
		GTK_SIGNAL_FUNC(guitl_ListClear) , 
		(gpointer) gui_cfg.formdata_list);

	button = guitl_pixmap_button(delete_xpm , NULL , gettext("Delete"));
	gtk_container_border_width(GTK_CONTAINER(button), 0);
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
		GTK_SIGNAL_FUNC(guitl_ListDeleteSelected) , 
		(gpointer) gui_cfg.formdata_list);

#ifdef _GTK_FEATURES_1_2
	/*** DRAG'N'DROP ***/
  	gtk_drag_dest_set(box , GTK_DEST_DEFAULT_ALL,
			dragtypes , NUM_ELEM(dragtypes) - 1 ,
               		GDK_ACTION_COPY | GDK_ACTION_MOVE);
  	gtk_signal_connect(GTK_OBJECT(box) ,
			"drag_data_received" ,
			GTK_SIGNAL_FUNC(gui_window_drop_url),
			NULL);
#endif

	box = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(tbox), box, TRUE, TRUE, 1);
	gtk_widget_show(box);

	urldlg.ext_frame = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(box), urldlg.ext_frame, FALSE, FALSE, 1);
	gtk_widget_show(urldlg.ext_frame);

	ptab = gtk_vbox_new(FALSE,5);
	gtk_container_add(GTK_CONTAINER(urldlg.ext_frame), ptab);
	gtk_widget_show(ptab);

	label = gtk_label_new(gettext("Query fields: "));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0 , 0.5);
	gtk_box_pack_start(GTK_BOX(ptab), label, FALSE, FALSE, 2);
	gtk_widget_show(label);

	swin = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_usize(swin , -1, 100);
	gtk_box_pack_start(GTK_BOX(ptab), swin, TRUE, TRUE, 2);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_widget_show(swin);

	urldlg.field_list = gtk_clist_new(3);
	gtk_clist_set_selection_mode(GTK_CLIST(urldlg.field_list), GTK_SELECTION_BROWSE);
	gtk_clist_set_column_title(GTK_CLIST(urldlg.field_list), 0 , gettext("Type"));
	gtk_clist_set_column_title(GTK_CLIST(urldlg.field_list), 1 , gettext("Name"));
	gtk_clist_set_column_title(GTK_CLIST(urldlg.field_list), 2 , gettext("Value"));
	gtk_clist_column_titles_show(GTK_CLIST(urldlg.field_list));
#ifdef _GTK_FEATURES_1_2
	gtk_clist_set_column_auto_resize(GTK_CLIST(urldlg.field_list) , 0 , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(urldlg.field_list) , 1 , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(urldlg.field_list) , 2 , TRUE);
#endif
	gtk_container_add(GTK_CONTAINER(swin), urldlg.field_list);
	gtk_widget_show(urldlg.field_list);

	gtk_signal_connect(GTK_OBJECT(urldlg.field_list) , "select_row" , 
			GTK_SIGNAL_FUNC(cfg_url_dlg_field_switch) , 
			(gpointer) &urldlg);

	prow = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(ptab), prow , FALSE, FALSE, 1);
	gtk_widget_show(prow);

	button = guitl_pixmap_button(append_xpm , NULL ,gettext("Append"));
	gtk_container_add(GTK_CONTAINER(prow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button), "clicked" ,
		GTK_SIGNAL_FUNC(cfg_url_dlg_field_append), &urldlg);

	button = guitl_pixmap_button(modify_xpm , NULL ,gettext("Modify"));
	gtk_container_add(GTK_CONTAINER(prow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button), "clicked" ,
		GTK_SIGNAL_FUNC(cfg_url_dlg_field_modify), &urldlg);

	button = guitl_pixmap_button(delete_xpm , NULL ,gettext("Delete"));
	gtk_container_add(GTK_CONTAINER(prow), button);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
		GTK_SIGNAL_FUNC(guitl_ListDeleteSelected) , 
		(gpointer) urldlg.field_list);

	prow = gtk_table_new(2,3, FALSE);
	gtk_box_pack_start(GTK_BOX(ptab), prow, FALSE, FALSE, 1);
	gtk_widget_show(prow);

	label = gtk_label_new(gettext("Type: "));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0 , 0.5);
	gtk_table_attach(GTK_TABLE(prow), label, 0, 1, 0, 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	gtk_widget_show(label);

	urldlg.field_type = gtk_option_menu_new();
	gtk_table_attach(GTK_TABLE(prow), urldlg.field_type, 1, 2, 0, 1, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	menu = gtk_menu_new();
	mi = gtk_menu_item_new_with_label("TEXT");
	gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)FORM_T_TEXT);
	gtk_menu_append(GTK_MENU(menu), mi);
	gtk_widget_show(mi);
	gtk_signal_connect(GTK_OBJECT(mi) , "activate" , 
		GTK_SIGNAL_FUNC(cfg_url_dlg_set_type) , (gpointer) &urldlg);
	mi = gtk_menu_item_new_with_label("FILE");
	gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)FORM_T_FILE);
	gtk_menu_append(GTK_MENU(menu), mi);
	gtk_widget_show(mi);
	gtk_signal_connect(GTK_OBJECT(mi) , "activate" , 
		GTK_SIGNAL_FUNC(cfg_url_dlg_set_type) , (gpointer) &urldlg);
	mi = gtk_menu_item_new_with_label("LONG TEXT");
	gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)FORM_T_TEXTAREA);
	gtk_menu_append(GTK_MENU(menu), mi);
	gtk_widget_show(mi);
	gtk_signal_connect(GTK_OBJECT(mi) , "activate" , 
		GTK_SIGNAL_FUNC(cfg_url_dlg_set_type) , (gpointer) &urldlg);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(urldlg.field_type), menu);
	gtk_widget_show(urldlg.field_type);

	urldlg.field_name = guitl_tab_add_entry(prow , gettext("Name: "), 0 , 1 , FALSE);

	label = gtk_label_new(gettext("Value: "));
	gtk_misc_set_alignment(GTK_MISC(label), 0.0 , 0.0);
	gtk_table_attach(GTK_TABLE(prow), label, 0, 1, 2, 3, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	gtk_widget_show(label);

	urldlg.tab = gtk_notebook_new();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(urldlg.tab), FALSE);
	gtk_table_attach(GTK_TABLE(prow), urldlg.tab, 1, 2, 2, 3, GTK_SHRINK | GTK_FILL, GTK_FILL, 2, 2);
	gtk_widget_show(urldlg.tab);

	label = gtk_label_new("TEXT");
	brow = gtk_vbox_new(FALSE, 1);
	gtk_widget_show(brow);
	gtk_notebook_append_page(GTK_NOTEBOOK(urldlg.tab), brow , label);

	urldlg.field_value_l = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(brow), urldlg.field_value_l , FALSE, FALSE , 1);
	gtk_widget_show(urldlg.field_value_l);

	label = gtk_label_new("FILE");
	brow = gtk_vbox_new(FALSE, 1);
	gtk_widget_show(brow);
	gtk_notebook_append_page(GTK_NOTEBOOK(urldlg.tab), brow , label);

	pbox = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(brow), pbox , FALSE, FALSE, 1);
	gtk_widget_show(pbox);

	urldlg.field_value_f = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(pbox), urldlg.field_value_f , TRUE, TRUE , 1);
	gtk_widget_show(urldlg.field_value_f);

	button = guitl_pixmap_button(browse_xpm , NULL ,gettext("Browse ..."));
	gtk_box_pack_start(GTK_BOX(pbox), button , FALSE, FALSE , 1);
	gtk_widget_show(button);

	gtk_signal_connect(GTK_OBJECT(button), "clicked" ,
		GTK_SIGNAL_FUNC(cfg_url_dlg_chose_file), urldlg.field_value_f);

	label = gtk_label_new("LONG TEXT");
	brow = gtk_table_new(2,2,FALSE);
	gtk_widget_show(brow);
	gtk_notebook_append_page(GTK_NOTEBOOK(urldlg.tab), brow , label);

	hadj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0));
	hsb = gtk_hscrollbar_new(hadj);
	gtk_table_attach(GTK_TABLE(brow), hsb, 0, 1, 1, 2,
		GTK_EXPAND | GTK_FILL, GTK_FILL, 0 , 0);
	gtk_widget_show(hsb);
 
	vadj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 0.0, 0.0, 0.0, 0.0));
	vsb = gtk_vscrollbar_new(vadj);
	gtk_table_attach(GTK_TABLE(brow), vsb, 1, 2, 0, 1,
		GTK_FILL, GTK_EXPAND | GTK_FILL, 0 , 0);
	gtk_widget_show(vsb);

	urldlg.field_value_t = gtk_text_new(hadj, vadj);
	gtk_widget_set_usize(urldlg.field_value_t, -1, 100);
	gtk_text_set_editable(GTK_TEXT(urldlg.field_value_t), TRUE);
	gtk_table_attach(GTK_TABLE(brow), urldlg.field_value_t, 0, 1, 0, 1,
		GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0 , 0);
	gtk_widget_show(urldlg.field_value_t);
}



/*** GRABER I ***/
static void cfgtab_graberI(notebook)
GtkWidget *notebook;
{
	GtkWidget *brow , *label , *box , *frame , *ptab;
	GList *ptr;
	int i;
	char *identity_combo[] = {
			"",
			PACKAGE"/"VERSION" ("HOSTTYPE")" ,
			"Mozilla/4.0 (compatible; "PACKAGE" "VERSION"; Linux 2.2.14 i486; X11)" ,
			"Mozilla/4.7 [en] (X11; Linux 2.2.14 i486; I)" ,
			"Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)" ,
			"Lynx/2.7 libWWW-FM/2.14" ,
			"NCSA_Mosaic/2.7b5 (X11;Linux 2.0.31 i486)  libwww/2.12 modified" ,
			"Opera/3.0" ,
			NULL ,
		};

	box = gtk_vbox_new(FALSE , 1);
	gtk_widget_show(box);
	label = gtk_label_new(gettext("Grabber I"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box , label);

	brow = gtk_table_new(1 , 2 , FALSE);
	gtk_box_pack_start (GTK_BOX (box), brow , FALSE, TRUE, 1);
	gtk_widget_show (brow);

	gui_cfg.cdir_label = guitl_tab_add_path_entry(brow , gettext("Cache Directory: ") ,
			0 , 0 , TRUE);

	brow = gtk_table_new(1 , 2 , FALSE);
	gtk_box_pack_start (GTK_BOX (box), brow , FALSE, TRUE, 1);
	gtk_widget_show (brow);

	gui_cfg.index_name = guitl_tab_add_entry(brow , gettext("Index file name: ") ,
			0 , 0 , FALSE);

	brow = gtk_table_new(1 , 2 , FALSE);
	gtk_box_pack_start (GTK_BOX (box), brow , FALSE, TRUE, 1);
	gtk_widget_show (brow);

	gui_cfg.store_name = guitl_tab_add_entry(brow , gettext("Store file name: ") ,
			0 , 0 , FALSE);

	brow = gtk_table_new(1 , 2 , FALSE);
	gtk_box_pack_start (GTK_BOX (box), brow , FALSE, TRUE, 1);
	gtk_widget_show (brow);

	label = gtk_label_new(gettext("Identity string: "));
	gtk_misc_set_alignment(GTK_MISC(label) , 0 , 0.5);
	gtk_table_attach(GTK_TABLE(brow), label, 0, 1, 0, 1, GTK_FILL | GTK_SHRINK, GTK_FILL, 1, 1);
	gtk_widget_show(label);

	label = gtk_combo_new();
	ptr = NULL;
	for (i = 0; identity_combo[i] ; i++)
		ptr = g_list_append(ptr, identity_combo[i]);
	gtk_combo_set_popdown_strings(GTK_COMBO(label), ptr);
	gtk_table_attach(GTK_TABLE(brow), label, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL , GTK_FILL, 1, 1);
	gtk_widget_show(label);
	gui_cfg.identity = GTK_COMBO(label)->entry;

#ifdef HAVE_DBOPEN
	brow = gtk_table_new(1 , 2 , FALSE);
	gtk_box_pack_start (GTK_BOX (box), brow , FALSE, TRUE, 1);
	gtk_widget_show (brow);

	gui_cfg.ns_cache_dir = guitl_tab_add_path_entry(brow , gettext("Netscape browser cache directory: ") ,
			0 , 0 , TRUE);
#endif

#ifdef WITH_TREE
	brow = gtk_table_new(1 , 2 , FALSE);
	gtk_box_pack_start (GTK_BOX (box), brow , FALSE, TRUE, 1);
	gtk_widget_show (brow);
	gui_cfg.browser_label = guitl_tab_add_entry(brow , gettext("Browser: ") , 0 , 0 , FALSE);
#endif

	brow = gtk_table_new(1 , 2 , FALSE);
	gtk_box_pack_start (GTK_BOX (box), brow , FALSE, TRUE, 1);
	gtk_widget_show (brow);
	gui_cfg.remind_cmd = guitl_tab_add_entry(brow , gettext("Reminder command: ") , 0 , 0 , FALSE);

	brow = gtk_table_new(1 , 2 , FALSE);
	gtk_box_pack_start (GTK_BOX (box), brow , FALSE, TRUE, 1);
	gtk_widget_show (brow);
	gui_cfg.post_cmd = guitl_tab_add_entry(brow , gettext("Post command: ") , 0 , 0 , FALSE);

/*********/
	frame = gtk_frame_new(NULL);
	gtk_widget_show(frame);
	gtk_box_pack_start (GTK_BOX (box), frame , FALSE, TRUE, 1);

	ptab = gtk_table_new (5 , 6 , FALSE);
	gtk_container_add (GTK_CONTAINER (frame), ptab);
	gtk_widget_show(ptab);

	gui_cfg.retry_label = guitl_tab_add_numentry(ptab , 
		gettext("How many times retry on fail: ") , 0 , 0 , USHRT_MAX);

	gui_cfg.redir_label = guitl_tab_add_numentry(ptab , 
		gettext("How many moved links to follow: ") , 0 , 1 , USHRT_MAX);

	gui_cfg.reget_label = guitl_tab_add_numentry(ptab , 
		gettext("How many times to reget file: ") , 0 , 2 , USHRT_MAX);

	gui_cfg.ddays_label = guitl_tab_add_numentry(ptab , 
		gettext("Document age before syncing with server: ") , 0 , 3 , USHRT_MAX);

	label = gtk_label_new(gettext(" days "));
	gtk_table_attach (GTK_TABLE (ptab), label , 2 , 3 , 3 , 4,
			GTK_SHRINK , GTK_FILL, 2 , 2);
	gtk_widget_show(label);

	gui_cfg.bufsize = guitl_tab_add_numentry(ptab , 
		gettext("Read buffer size: ") , 0 , 4 , 10000);
	gtk_adjustment_clamp_page(
		GTK_ADJUSTMENT(GTK_SPIN_BUTTON(gui_cfg.bufsize)->adjustment) ,
		1.0 , 1024.0);

	label = gtk_label_new(gettext(" kB "));
	gtk_table_attach (GTK_TABLE (ptab), label , 2 , 3 , 4 , 5,
			GTK_SHRINK , GTK_FILL, 2 , 2);
	gtk_widget_show(label);


	gui_cfg.sleep_label = guitl_tab_add_numentry(ptab , 
		gettext("Sleep time between transfers: ") , 3 , 0 , INT_MAX);

	label = gtk_label_new(gettext(" sec."));
	gtk_table_attach (GTK_TABLE (ptab), label , 5 , 6 , 0 , 1,
			GTK_SHRINK , GTK_FILL, 2 , 2);
	gtk_widget_show(label);


	gui_cfg.rollback_label = guitl_tab_add_numentry(ptab , 
		gettext("Rollback amount on reget: ") , 3 , 1 , INT_MAX);

	label = gtk_label_new(gettext(" bytes"));
	gtk_table_attach (GTK_TABLE (ptab), label , 5 , 6 , 1 , 2,
			GTK_SHRINK , GTK_FILL, 2 , 2);
	gtk_widget_show(label);

	gui_cfg.trans_quota = guitl_tab_add_numentry(ptab , 
		gettext("Transfer quota: ") , 3 , 2 , INT_MAX);

	label = gtk_label_new(gettext(" kB"));
	gtk_table_attach (GTK_TABLE (ptab), label , 5 , 6 , 2 , 3,
			GTK_SHRINK , GTK_FILL, 2 , 2);
	gtk_widget_show(label);

	gui_cfg.file_quota = guitl_tab_add_numentry(ptab , 
		gettext("File size quota: ") , 3 , 3 , INT_MAX);

	label = gtk_label_new(gettext(" kB"));
	gtk_table_attach (GTK_TABLE (ptab), label , 5 , 6 , 3 , 4,
			GTK_SHRINK , GTK_FILL, 2 , 2);
	gtk_widget_show(label);

#if defined HAVE_FSTATFS || HAVE_FSTATVFS
	gui_cfg.fs_quota = guitl_tab_add_numentry(ptab , 
		gettext("Filesystem freespace quota: ") , 3 , 4 , INT_MAX);

	label = gtk_label_new(gettext(" kB"));
	gtk_table_attach (GTK_TABLE (ptab), label , 5 , 6 , 4 , 5,
			GTK_SHRINK , GTK_FILL, 2 , 2);
	gtk_widget_show(label);
#endif

}

/*** GRABER II ***/
static void HtmlswSens(w , data)
GtkWidget *w;
gpointer data;
{
	gboolean act = GTK_TOGGLE_BUTTON(w)->active;

	gtk_widget_set_sensitive(gui_cfg.all_to_local , act);
	gtk_widget_set_sensitive(gui_cfg.sel_to_local , act);
	gtk_widget_set_sensitive(gui_cfg.url_to_local , act);
	gtk_widget_set_sensitive(gui_cfg.all_to_remote , act);
}

static void cfgtab_graberII(notebook)
GtkWidget *notebook;
{
	GtkWidget *brow , *label , *frame , *ptab , *brow2;
	GtkWidget *menu, *mi;
	GSList *rg;
	int i;

	brow = gtk_hbox_new(FALSE , 5);
	gtk_widget_show(brow);
	label = gtk_label_new (gettext("Grabber II"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), brow , label);

	frame = gtk_frame_new(gettext("HTML document URL rewriting"));
	gtk_widget_show(frame);
	gtk_box_pack_start (GTK_BOX (brow), frame , FALSE, TRUE, 1);
	
	brow2 = gtk_vbox_new(0 , 1);
	gtk_widget_show(brow2);
	gtk_container_add(GTK_CONTAINER(frame) , brow2);

	gui_cfg.noreloc_sw = gtk_check_button_new_with_label(
		gettext("Rewrite URLs inside HTML doc."));
	gtk_widget_show(gui_cfg.noreloc_sw);
	gtk_box_pack_start (GTK_BOX (brow2), gui_cfg.noreloc_sw, FALSE, TRUE, 2);
	gtk_signal_connect(GTK_OBJECT(gui_cfg.noreloc_sw) , "toggled" ,
		GTK_SIGNAL_FUNC(HtmlswSens) , NULL);

	rg = NULL;
	gui_cfg.url_to_local = gtk_radio_button_new_with_label( rg ,
		gettext("Rewrite URLs to local when stored locally"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(gui_cfg.url_to_local));
	gtk_widget_show(gui_cfg.url_to_local);
	gtk_box_pack_start (GTK_BOX (brow2), gui_cfg.url_to_local, FALSE, TRUE, 1);

	gui_cfg.sel_to_local = gtk_radio_button_new_with_label(rg ,
		gettext("Rewrite all suitable URLs to local"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(gui_cfg.sel_to_local));
	gtk_widget_show(gui_cfg.sel_to_local);
	gtk_box_pack_start (GTK_BOX (brow2), gui_cfg.sel_to_local, FALSE, TRUE, 1);

	gui_cfg.all_to_local = gtk_radio_button_new_with_label(rg ,
		gettext("Rewrite all URLs to local immediately"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(gui_cfg.all_to_local));
	gtk_widget_show(gui_cfg.all_to_local);
	gtk_box_pack_start (GTK_BOX (brow2), gui_cfg.all_to_local, FALSE, TRUE, 1);

	gui_cfg.all_to_remote = gtk_radio_button_new_with_label( rg ,
		gettext("Rewrite all URLs to remote immediately"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(gui_cfg.all_to_remote));
	gtk_widget_show(gui_cfg.all_to_remote);
	gtk_box_pack_start (GTK_BOX (brow2), gui_cfg.all_to_remote, FALSE, TRUE, 1);

/***/
	frame = gtk_frame_new(gettext("Misc settings"));
	gtk_widget_show(frame);
	gtk_box_pack_start (GTK_BOX (brow), frame, FALSE , TRUE, 1);

	brow = gtk_vbox_new(0 , 1);
	gtk_widget_show(brow);
	gtk_container_add (GTK_CONTAINER (frame), brow);
	
	gui_cfg.ptime = gtk_check_button_new_with_label(
		gettext("Preserve document modification time"));
	gtk_widget_show(gui_cfg.ptime);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.ptime, FALSE, TRUE, 1);

	gui_cfg.preserve_perm = gtk_check_button_new_with_label(
		gettext("Preserve FTP document permissions"));
	gtk_widget_show(gui_cfg.preserve_perm);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.preserve_perm, FALSE, TRUE, 1);

	gui_cfg.preserve_links = gtk_check_button_new_with_label(
		gettext("Preserve FTP symbolic links"));
	gtk_widget_show(gui_cfg.preserve_links);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.preserve_links, FALSE, TRUE, 1);

	gui_cfg.freget_sw = gtk_check_button_new_with_label(
		gettext("Whole reget when partial not supported"));
	gtk_widget_show(gui_cfg.freget_sw);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.freget_sw, FALSE, TRUE, 1);


	gui_cfg.enc_sw = gtk_check_button_new_with_label(
		gettext("Use gzip encoding for transfer"));
	gtk_widget_show(gui_cfg.enc_sw);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.enc_sw, FALSE, TRUE, 1);

	gui_cfg.oldrm_sw = gtk_check_button_new_with_label(
		gettext("Remove improper documents"));
	gtk_widget_show(gui_cfg.oldrm_sw);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.oldrm_sw, FALSE, TRUE, 1);

	gui_cfg.check_size = gtk_check_button_new_with_label(
		gettext("Check transferred size of document"));
	gtk_widget_show(gui_cfg.check_size);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.check_size , FALSE, TRUE, 1);

	gui_cfg.store_index = gtk_check_button_new_with_label(
		gettext("Store directory URLs as index files"));
	gtk_widget_show(gui_cfg.store_index);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.store_index , FALSE, TRUE, 1);

	gui_cfg.enable_js = gtk_check_button_new_with_label(
		gettext("Download javascript script sources"));
	gtk_widget_show(gui_cfg.enable_js);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.enable_js , FALSE, TRUE, 1);

	gui_cfg.enable_info = gtk_check_button_new_with_label(
		gettext("Store info files with each document"));
	gtk_widget_show(gui_cfg.enable_info);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.enable_info , FALSE, TRUE, 1);

	gui_cfg.auto_referer = gtk_check_button_new_with_label(
		gettext("Send self URL as Referer for starting URLs"));
	gtk_widget_show(gui_cfg.auto_referer);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.auto_referer , FALSE, TRUE, 1);

	gui_cfg.send_if_range = gtk_check_button_new_with_label(
		gettext("Send If-Range header field when regeting"));
	gtk_widget_show(gui_cfg.send_if_range);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.send_if_range , FALSE, TRUE, 1);

	gui_cfg.show_time = gtk_check_button_new_with_label(
		gettext("Show time of start and end of downloading"));
	gtk_widget_show(gui_cfg.show_time);
	gtk_box_pack_start (GTK_BOX (brow), gui_cfg.show_time , FALSE, TRUE, 1);

	ptab = gtk_table_new (2 , 1 , FALSE);
	gtk_box_pack_start (GTK_BOX (brow), ptab , FALSE, FALSE, 1);
	gtk_widget_show(ptab);

	label = gtk_label_new(gettext("URL scheduling strategy: "));
	gtk_table_attach (GTK_TABLE (ptab), label , 0 , 1 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  1 , 1);
	gtk_widget_show(label);

	gui_cfg.scheduling_strategie = gtk_option_menu_new();

	menu = gtk_menu_new();

	for (i = 0 ; i < SSTRAT_LAST ; i++)
	{
		mi = gtk_menu_item_new_with_label(get_strategie_label(i));
		gtk_object_set_user_data(GTK_OBJECT(mi) , (gpointer)i);
		gtk_menu_append (GTK_MENU (menu), mi);
		gtk_widget_show(mi);
	}

	gtk_option_menu_set_menu (GTK_OPTION_MENU (gui_cfg.scheduling_strategie), menu);

	gtk_table_attach (GTK_TABLE (ptab), gui_cfg.scheduling_strategie ,
			1 , 2 , 0 , 1 , GTK_FILL , GTK_FILL,  1 , 1);
	gtk_widget_show(gui_cfg.scheduling_strategie);

	ptab = gtk_table_new (2 , 1 , FALSE);
	gtk_box_pack_start (GTK_BOX (brow), ptab , FALSE, FALSE, 1);
	gtk_widget_show(ptab);

	gui_cfg.hash_size = guitl_tab_add_numentry(ptab ,
		gettext("Hash tables size: ") , 3 , 3 , SHRT_MAX);

#ifdef HAVE_MT
	ptab = gtk_table_new (2 , 1 , FALSE);
	gtk_box_pack_start (GTK_BOX (brow), ptab , FALSE, FALSE, 1);
	gtk_widget_show(ptab);

#ifdef PTHREAD_THREADS_MAX
	gui_cfg.nthr = guitl_tab_add_numentry(ptab ,
		gettext("Number of downloading threads: ") , 3 , 3 , PTHREAD_THREADS_MAX - 3);
#else
#ifdef _POSIX_THREADS_THREAD_MAX
	gui_cfg.nthr = guitl_tab_add_numentry(ptab ,
		gettext("Number of downloading threads: ") , 3 , 3 , _POSIX_THREADS_THREAD_MAX - 3);
#else
	gui_cfg.nthr = guitl_tab_add_numentry(ptab ,
		gettext("Number of downloading threads: ") , 3 , 3 , 61);
#endif
#endif

#endif
}


/*** NET ***/
static void cfgtab_net(notebook)
GtkWidget *notebook;
{
	GtkWidget *col , *brow , *box , *frame, *label , *rb;
	GSList *rg;

	box = gtk_vbox_new (0,5);
	gtk_widget_show(box);
	label = gtk_label_new (gettext("Net"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box , label);

	col = gtk_table_new(7 , 3 , FALSE);
	gtk_widget_show(col);
	gtk_container_add (GTK_CONTAINER (box), col);

	frame = gtk_frame_new(gettext("Allowed protocols"));
	gtk_widget_show(frame);
	gtk_table_attach (GTK_TABLE (col), frame , 1 , 2 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	brow = gtk_table_new(3 , 2 , FALSE);
	gtk_widget_show(brow);
	gtk_container_add (GTK_CONTAINER (frame), brow);

	gui_cfg.http_sw = gtk_check_button_new_with_label(gettext("HTTP"));
	gtk_widget_show(gui_cfg.http_sw);
	gtk_table_attach (GTK_TABLE (brow), gui_cfg.http_sw , 0 , 1 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	gui_cfg.ftp_sw = gtk_check_button_new_with_label(gettext("FTP"));
	gtk_widget_show(gui_cfg.ftp_sw);
	gtk_table_attach (GTK_TABLE (brow), gui_cfg.ftp_sw  , 0 , 1 , 1 , 2 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	gui_cfg.gopher_sw = gtk_check_button_new_with_label(gettext("Gopher"));
	gtk_widget_show(gui_cfg.gopher_sw);
	gtk_table_attach (GTK_TABLE (brow), gui_cfg.gopher_sw , 1 , 2 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  5 , 5);

#ifdef USE_SSL
	gui_cfg.https_sw = gtk_check_button_new_with_label(gettext("HTTPS"));
	gtk_widget_show(gui_cfg.https_sw);
	gtk_table_attach (GTK_TABLE (brow), gui_cfg.https_sw , 1 , 2 , 1 , 2 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	gui_cfg.ftps_sw = gtk_check_button_new_with_label(gettext("FTPS"));
	gtk_widget_show(gui_cfg.ftps_sw);
	gtk_table_attach (GTK_TABLE (brow), gui_cfg.ftps_sw , 2 , 3 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  5 , 5);
#endif

	frame = gtk_frame_new(gettext("FTP data connection type "));
	gtk_widget_show(frame);
	gtk_table_attach (GTK_TABLE (col), frame , 0 , 1 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	brow = gtk_table_new(2 , 1 , FALSE);
	gtk_widget_show(brow);
	gtk_container_add (GTK_CONTAINER (frame), brow);

	rb = gtk_radio_button_new_with_label(NULL , gettext("Active"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_table_attach (GTK_TABLE (brow), rb , 0 , 1 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  5 , 5);
	gui_cfg.ftpmodegr[1] = rb;

	rb = gtk_radio_button_new_with_label(rg , gettext("Passive"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_table_attach (GTK_TABLE (brow), rb , 0 , 1 , 1 , 2 ,
			GTK_FILL , GTK_FILL,  5 , 5);
	gui_cfg.ftpmodegr[0] = rb;

	label = gtk_label_new(gettext("Communication timeout: "));
	gtk_table_attach (GTK_TABLE (col), label , 0 , 1 , 1 , 2 ,
			GTK_FILL , GTK_FILL,  2 , 5);
	gtk_misc_set_alignment(GTK_MISC(label) , 0.0 , 0.5);
	gtk_widget_show (label);

	gui_cfg.timeout_label = gtk_spin_button_new(
		(GtkAdjustment *)gtk_adjustment_new(0.0 , 0.0 , 9999 , 1.0 , 10.0 , 0.0) ,
		0, 0);
	gtk_table_attach (GTK_TABLE (col), gui_cfg.timeout_label , 1, 2 , 1 , 2 ,
			GTK_FILL , GTK_FILL ,  2 , 5);
	gtk_widget_show (gui_cfg.timeout_label);

	label = gtk_label_new(gettext(" min."));
	gtk_table_attach (GTK_TABLE (col), label , 2 , 3 , 1 , 2 ,
			GTK_FILL , GTK_FILL,  2 , 5);
	gtk_misc_set_alignment(GTK_MISC(label) , 0.0 , 0.5);
	gtk_widget_show (label);

	label = gtk_label_new(gettext("Maximal transfer rate: "));
	gtk_table_attach (GTK_TABLE (col), label , 0 , 1 , 3 , 4 ,
			GTK_FILL , GTK_FILL,  2 , 5);
	gtk_misc_set_alignment(GTK_MISC(label) , 0.0 , 0.5);
	gtk_widget_show (label);

	gui_cfg.maxrate = gtk_spin_button_new(
		(GtkAdjustment *)gtk_adjustment_new(0.0 , 0.0 , 1000000.0 , 1.0 , 10.0 , 0.0) ,
		0, 3);
	gtk_table_attach (GTK_TABLE (col), gui_cfg.maxrate , 1, 2 , 3 , 4 ,
			GTK_FILL , GTK_FILL ,  2 , 5);
	gtk_widget_show (gui_cfg.maxrate);

	label = gtk_label_new(gettext(" kB/s"));
	gtk_table_attach (GTK_TABLE (col), label , 2 , 3 , 3 , 4 ,
			GTK_FILL , GTK_FILL,  2 , 5);
	gtk_misc_set_alignment(GTK_MISC(label) , 0.0 , 0.5);
	gtk_widget_show (label);

	label = gtk_label_new(gettext("Minimal transfer rate: "));
	gtk_table_attach (GTK_TABLE (col), label , 0 , 1 , 4 , 5 ,
			GTK_FILL , GTK_FILL,  2 , 5);
	gtk_misc_set_alignment(GTK_MISC(label) , 0.0 , 0.5);
	gtk_widget_show (label);

	gui_cfg.minrate = gtk_spin_button_new(
		(GtkAdjustment *)gtk_adjustment_new(0.0 , 0.0 , 1000000.0 , 1.0 , 10.0 , 0.0) ,
		0, 3);
	gtk_table_attach (GTK_TABLE (col), gui_cfg.minrate , 1, 2 , 4 , 5 ,
			GTK_FILL , GTK_FILL ,  2 , 5);
	gtk_widget_show (gui_cfg.minrate);

	label = gtk_label_new(gettext(" kB/s"));
	gtk_table_attach (GTK_TABLE (col), label , 2 , 3 , 4 , 5 ,
			GTK_FILL , GTK_FILL,  2 , 5);
	gtk_misc_set_alignment(GTK_MISC(label) , 0.0 , 0.5);
	gtk_widget_show(label);

	label = gtk_label_new(gettext("Local interface address: "));
	gtk_table_attach(GTK_TABLE(col), label , 0 , 1 , 5 , 6 ,
			GTK_FILL , GTK_FILL,  2 , 5);
	gtk_misc_set_alignment(GTK_MISC(label) , 0.0 , 0.5);
	gtk_widget_show(label);

	gui_cfg.local_ip = gtk_entry_new();
	gtk_table_attach(GTK_TABLE(col), gui_cfg.local_ip , 1, 3 , 5 , 6 ,
			GTK_FILL , GTK_FILL ,  2 , 5);
	gtk_widget_show(gui_cfg.local_ip);

	label = gtk_label_new(gettext("Additional HTTP headers: "));
	gtk_table_attach (GTK_TABLE (col), label , 0 , 1 , 6 , 7 ,
			GTK_FILL , GTK_FILL,  2 , 5);
	gtk_misc_set_alignment(GTK_MISC(label) , 0.0 , 0.5);
	gtk_widget_show (label);

	gui_cfg.http_headers = gtk_entry_new();
	gtk_table_attach (GTK_TABLE (col), gui_cfg.http_headers , 1, 3 , 6 , 7 ,
			GTK_FILL , GTK_FILL ,  2 , 5);
	gtk_widget_show (gui_cfg.http_headers);

	gui_cfg.ftp_list = gtk_check_button_new_with_label(
		gettext("Use wide listing of FTP directories"));
	gtk_widget_show(gui_cfg.ftp_list);
	gtk_table_attach (GTK_TABLE (col), gui_cfg.ftp_list, 0 , 3 , 7 , 8 ,
			GTK_FILL , GTK_FILL ,  5 , 5);

	gui_cfg.use_http11 = gtk_check_button_new_with_label(
		gettext("Use HTTP/1.1 protocol for HTTP communication"));
	gtk_widget_show(gui_cfg.use_http11);
	gtk_table_attach (GTK_TABLE (col), gui_cfg.use_http11, 0 , 3 , 8 , 9 ,
			GTK_FILL , GTK_FILL ,  5 , 5);
}


/*** PROXY ***/
static void cfgtab_proxy(notebook)
GtkWidget *notebook;
{
	GtkWidget *col , *brow , *label , *box , *frame;

	box = gtk_vbox_new (0,5);
	gtk_widget_show(box);
	label = gtk_label_new (gettext("Proxy"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box , label);

	col = gtk_table_new(3 , 2 , FALSE);
	gtk_widget_show(col);
	gtk_container_add (GTK_CONTAINER (box), col);
	frame = gtk_frame_new(gettext("Gopher proxy"));
	gtk_widget_show(frame);
	gtk_table_attach (GTK_TABLE (col), frame , 0 , 1 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	brow = gtk_table_new(3 , 2 , FALSE);
	gtk_widget_show(brow);
	gtk_container_add (GTK_CONTAINER (frame), brow);


	gui_cfg.gopher_proxyh_label = guitl_tab_add_entry(brow , 
		gettext("Host: ") , 0 , 0 , FALSE);

	gui_cfg.gopher_proxyp_label = guitl_tab_add_numentry(brow , 
		gettext("Port: ") , 0 , 1 , USHRT_MAX);


	gui_cfg.gopher_httpgw = gtk_check_button_new_with_label(
		gettext("Gopher via HTTP proxy"));
	gtk_widget_show(gui_cfg.gopher_httpgw);
	gtk_table_attach (GTK_TABLE (brow), gui_cfg.gopher_httpgw , 0 , 2 , 2 , 3 ,
			GTK_FILL , GTK_FILL,  5 , 5);


	frame = gtk_frame_new(gettext("FTP proxy"));
	gtk_widget_show(frame);
	gtk_table_attach (GTK_TABLE (col), frame , 1 , 2 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	brow = gtk_table_new(4 , 2 , FALSE);
	gtk_widget_show(brow);
	gtk_container_add (GTK_CONTAINER (frame), brow);


	gui_cfg.ftp_proxyh_label = guitl_tab_add_entry(brow , 
		gettext("Host: ") , 0 , 0 , FALSE);

	gui_cfg.ftp_proxyp_label = guitl_tab_add_numentry(brow , 
		gettext("Port: ") , 0 , 1 , USHRT_MAX);

	gui_cfg.ftp_httpgw = gtk_check_button_new_with_label(
		gettext("FTP via HTTP proxy"));
	gtk_widget_show(gui_cfg.ftp_httpgw);
	gtk_table_attach (GTK_TABLE (brow), gui_cfg.ftp_httpgw , 0 , 2 , 2 , 3 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	gui_cfg.ftp_dirtyp = gtk_check_button_new_with_label(
		gettext("FTP via HTTP tunneling proxy"));
	gtk_widget_show(gui_cfg.ftp_dirtyp);
	gtk_table_attach (GTK_TABLE (brow), gui_cfg.ftp_dirtyp , 0 , 2 , 3 , 4 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	frame = gtk_frame_new(gettext("HTTP proxy"));
	gtk_widget_show(frame);
	gtk_table_attach (GTK_TABLE (col), frame , 0 , 1 , 1 , 2 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	brow = guitl_new_edit_list(&gui_cfg.http_proxy_list , &gui_cfg.http_proxyh_label ,
		gettext("Proxy: ") , NULL , NULL , NULL , NULL , FALSE);

	gtk_container_add(GTK_CONTAINER(frame), brow);


#ifdef USE_SSL
	frame = gtk_frame_new(gettext("SSL proxy"));
	gtk_widget_show(frame);
	gtk_table_attach (GTK_TABLE (col), frame , 1 , 2 , 1 , 2 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	brow = gtk_table_new(2 , 2 , FALSE);
	gtk_widget_show(brow);
	gtk_container_add (GTK_CONTAINER (frame), brow);


	gui_cfg.ssl_proxyh_label = guitl_tab_add_entry(brow , 
		gettext("Host: ") , 0 , 0 , FALSE);

	gui_cfg.ssl_proxyp_label = guitl_tab_add_numentry(brow , 
		gettext("Port: ") , 0 , 1 , USHRT_MAX);

#endif

	gui_cfg.cache_sw = gtk_check_button_new_with_label(
		gettext("Allow caching of documents"));
	gtk_widget_show(gui_cfg.cache_sw);
	gtk_table_attach (GTK_TABLE (col), gui_cfg.cache_sw, 0 , 1 , 2 , 3 ,
			GTK_FILL , GTK_FILL ,  5 , 5);
}


/*** LANGUAGES ***/
static void cfgtab_lang(notebook)
GtkWidget *notebook;
{
	GtkWidget *col , *brow , *box , *swin , *button , *label , *arrow;
	GtkWidget *dbutton , *abutton , *cbutton, *mbutton;
	static listanfo p;
	static listanfo e;

	box = gtk_vbox_new (0,5);
	gtk_widget_show(box);
	label = gtk_label_new (gettext("Languages"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box , label);

	brow = gtk_table_new(1 , 3 , FALSE);
	gtk_widget_show(brow);
	gtk_container_add (GTK_CONTAINER (box), brow);

	swin = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_table_attach_defaults (GTK_TABLE (brow), swin , 0 , 1, 0, 1);
	gtk_widget_show (swin);

	gui_cfg.langlist = gtk_clist_new (1);
#ifdef _GTK_FEATURES_1_2
	gtk_clist_set_reorderable(GTK_CLIST(gui_cfg.langlist) , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(gui_cfg.langlist) , 0 , TRUE);
#endif
	gtk_clist_set_selection_mode (GTK_CLIST (gui_cfg.langlist), GTK_SELECTION_BROWSE);
	gtk_container_add(GTK_CONTAINER(swin), gui_cfg.langlist);
	gtk_widget_show (gui_cfg.langlist);

	col = gtk_vbox_new(FALSE , 20);
	gtk_widget_show(col);
	gtk_table_attach (GTK_TABLE (brow), col , 1 , 2 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  10 , 10);


	arrow = gtk_arrow_new(GTK_ARROW_RIGHT , GTK_SHADOW_OUT);
	gtk_widget_show(arrow);
	button = gtk_button_new();
	gtk_container_add(GTK_CONTAINER(button) , arrow);
	gtk_widget_show(button);
	gtk_widget_set_usize(button , 25 , 25);
	gtk_container_add (GTK_CONTAINER (col), button);

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListDeleteSelected) , 
			(gpointer) gui_cfg.langlist);

	arrow = gtk_arrow_new(GTK_ARROW_LEFT , GTK_SHADOW_OUT);
	gtk_widget_show(arrow);
	button = gtk_button_new();
	gtk_container_add(GTK_CONTAINER(button) , arrow);
	gtk_widget_show(button);
	gtk_widget_set_usize(button , 25 , 25);
	gtk_container_add (GTK_CONTAINER (col), button);

	col = guitl_new_edit_list(&gui_cfg.alanglist , &gui_cfg.alang_entry ,
		gettext("Other lang. code: ") , &dbutton , &mbutton , &cbutton , &abutton , FALSE);
	gtk_table_attach_defaults (GTK_TABLE (brow), col , 2 , 3 , 0, 1);

	gtk_signal_connect(GTK_OBJECT(dbutton) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListDeleteSelected) , 
			(gpointer) gui_cfg.langlist);

	e.entry = gui_cfg.alang_entry;
	e.list = gui_cfg.langlist;

	gtk_signal_connect(GTK_OBJECT(abutton) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListInsertEntry) , 
			(gpointer) &e);

	gtk_signal_connect(GTK_OBJECT(gui_cfg.alang_entry) , "activate" , 
			GTK_SIGNAL_FUNC(guitl_ListInsertEntry) , 
			(gpointer) &e);

	gtk_signal_connect(GTK_OBJECT(mbutton) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListModifyEntry) , 
			(gpointer) &e);

	gtk_signal_connect(GTK_OBJECT(cbutton) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListClear) , 
			(gpointer) gui_cfg.langlist);

	gtk_signal_connect(GTK_OBJECT(gui_cfg.langlist) , "select_row" , 
			GTK_SIGNAL_FUNC(guitl_ListCopyToEntry) , 
			(gpointer) gui_cfg.alang_entry);

	p.entry = gui_cfg.alanglist;
	p.list = gui_cfg.langlist;

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
			GTK_SIGNAL_FUNC(LangListInsertList) , 
			(gpointer) &p);

	FILL_LIST(gui_cfg.alanglist , iso_lang);
}

/*** CHARACTER-SET ***/
static void cfgtab_charset(notebook)
GtkWidget *notebook;
{
	GtkWidget *col , *brow , *box , *swin , *button , *label , *arrow;
	GtkWidget *dbutton , *abutton , *mbutton , *cbutton;
	static listanfo p;
	static listanfo e;

	box = gtk_vbox_new (0,5);
	gtk_widget_show(box);
	label = gtk_label_new (gettext("Character Sets"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box , label);

	brow = gtk_table_new(1 , 3 , FALSE);
	gtk_widget_show(brow);
	gtk_container_add (GTK_CONTAINER (box), brow);

	swin = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_table_attach_defaults (GTK_TABLE (brow), swin , 0 , 1, 0, 1);
	gtk_widget_show (swin);

	gui_cfg.charset_list = gtk_clist_new (1);
#ifdef _GTK_FEATURES_1_2
	gtk_clist_set_reorderable(GTK_CLIST(gui_cfg.charset_list) , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(gui_cfg.charset_list) , 0 , TRUE);
#endif
	gtk_clist_set_selection_mode (GTK_CLIST (gui_cfg.charset_list), GTK_SELECTION_BROWSE);
	gtk_container_add(GTK_CONTAINER(swin), gui_cfg.charset_list);
	gtk_widget_show (gui_cfg.charset_list);

	col = gtk_vbox_new(FALSE , 20);
	gtk_widget_show(col);
	gtk_table_attach (GTK_TABLE (brow), col , 1 , 2 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  10 , 10);


	arrow = gtk_arrow_new(GTK_ARROW_RIGHT , GTK_SHADOW_OUT);
	gtk_widget_show(arrow);
	button = gtk_button_new();
	gtk_container_add(GTK_CONTAINER(button) , arrow);
	gtk_widget_show(button);
	gtk_widget_set_usize(button , 25 , 25);
	gtk_container_add (GTK_CONTAINER (col), button);

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListDeleteSelected) , 
			(gpointer) gui_cfg.charset_list);

	arrow = gtk_arrow_new(GTK_ARROW_LEFT , GTK_SHADOW_OUT);
	gtk_widget_show(arrow);
	button = gtk_button_new();
	gtk_container_add(GTK_CONTAINER(button) , arrow);
	gtk_widget_show(button);
	gtk_widget_set_usize(button , 25 , 25);
	gtk_container_add (GTK_CONTAINER (col), button);

	col = guitl_new_edit_list(&gui_cfg.acharset_list , &gui_cfg.acharset_entry ,
		gettext("Other Charset code: ") , &dbutton , &mbutton , &cbutton , &abutton , FALSE);
	gtk_table_attach_defaults (GTK_TABLE (brow), col , 2 , 3 , 0, 1);

	gtk_signal_connect(GTK_OBJECT(dbutton) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListDeleteSelected) , 
			(gpointer) gui_cfg.charset_list);

	e.entry = gui_cfg.acharset_entry;
	e.list = gui_cfg.charset_list;

	gtk_signal_connect(GTK_OBJECT(abutton) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListInsertEntry) , 
			(gpointer) &e);

	gtk_signal_connect(GTK_OBJECT(gui_cfg.acharset_entry) , "activate" , 
			GTK_SIGNAL_FUNC(guitl_ListInsertEntry) , 
			(gpointer) &e);

	gtk_signal_connect(GTK_OBJECT(mbutton) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListModifyEntry) , 
			(gpointer) &e);

	gtk_signal_connect(GTK_OBJECT(cbutton) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListClear) , 
			(gpointer) gui_cfg.charset_list);

	gtk_signal_connect(GTK_OBJECT(gui_cfg.charset_list) , "select_row" , 
			GTK_SIGNAL_FUNC(guitl_ListCopyToEntry) , 
			(gpointer) gui_cfg.acharset_entry);

	p.entry = gui_cfg.acharset_list;
	p.list = gui_cfg.charset_list;

	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_ListInsertList) , 
			(gpointer) &p);

	FILL_LIST(gui_cfg.acharset_list , char_sets);
}


/*** AUTH ***/
static void cfgtab_auth(notebook)
GtkWidget *notebook;
{
	GtkWidget *col , *label , *box , *frame , *ptab ,
		*cpom , *rb;
	GSList *rg;

	box = gtk_vbox_new (0,5);
	gtk_widget_show(box);
	label = gtk_label_new (gettext("Auth"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box , label);

	col = gtk_table_new(5 , 2 , FALSE);
	gtk_widget_show(col);
	gtk_container_add (GTK_CONTAINER (box), col);

	frame = gtk_frame_new(gettext("HTTP user authentification"));
	gtk_widget_show(frame);
	gtk_table_attach (GTK_TABLE (col), frame , 0 , 2 , 0 , 1 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	ptab = gtk_table_new(3 , 3 , FALSE);
	gtk_widget_show(ptab);
	gtk_container_add (GTK_CONTAINER (frame), ptab);

	gui_cfg.auth_label = guitl_tab_add_entry(ptab , 
		gettext("Name: ") , 0 , 0 , FALSE);

	gui_cfg.pass_label = guitl_tab_add_entry(ptab , 
		gettext("Password: ") , 0 , 1 , TRUE);

	gui_cfg.auth_reuse_nonce = gtk_check_button_new_with_label(
		gettext("Reuse HTTP Digest access nonce"));
	gtk_widget_show(gui_cfg.auth_reuse_nonce);
	gtk_table_attach (GTK_TABLE (ptab), gui_cfg.auth_reuse_nonce , 0 , 2 , 2 , 3 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	frame = gtk_frame_new(gettext("Scheme"));
	gtk_widget_show(frame);
	gtk_table_attach (GTK_TABLE (ptab), frame , 2 , 3 , 0 , 3 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	cpom = gtk_vbox_new(TRUE , 5);
	gtk_widget_show(cpom);
	gtk_container_add (GTK_CONTAINER (frame), cpom);

	rb = gtk_radio_button_new_with_label(NULL , gettext("User auth. scheme"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_container_add (GTK_CONTAINER (cpom), rb);
	gui_cfg.http_schgr[0] = rb;

	rb = gtk_radio_button_new_with_label(rg , gettext("Base auth. scheme"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_container_add (GTK_CONTAINER (cpom), rb);
	gui_cfg.http_schgr[1] = rb;

	rb = gtk_radio_button_new_with_label(rg , gettext("Digest auth. scheme"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_container_add (GTK_CONTAINER (cpom), rb);
	gui_cfg.http_schgr[2] = rb;

	frame = gtk_frame_new(gettext("HTTP proxy user authentification"));
	gtk_widget_show(frame);
	gtk_table_attach (GTK_TABLE (col), frame , 0 , 2 , 1 , 2 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	ptab = gtk_table_new(3 , 3 , FALSE);
	gtk_widget_show(ptab);
	gtk_container_add (GTK_CONTAINER (frame), ptab);

	gui_cfg.proxy_auth_label = guitl_tab_add_entry(ptab , 
		gettext("Name: ") , 0 , 0 , FALSE);

	gui_cfg.proxy_pass_label = guitl_tab_add_entry(ptab , 
		gettext("Password: ") , 0 , 1 , TRUE);

	gui_cfg.auth_reuse_proxy_nonce = gtk_check_button_new_with_label(
		gettext("Reuse HTTP Digest access nonce"));
	gtk_widget_show(gui_cfg.auth_reuse_proxy_nonce);
	gtk_table_attach (GTK_TABLE (ptab), gui_cfg.auth_reuse_proxy_nonce , 0 , 2 , 2 , 3 ,
			GTK_FILL , GTK_FILL,  5 , 5);
	
	frame = gtk_frame_new(gettext("Scheme"));
	gtk_widget_show(frame);
	gtk_table_attach (GTK_TABLE (ptab), frame , 2 , 3 , 0 , 3 ,
			GTK_FILL , GTK_FILL,  5 , 5);

	cpom = gtk_vbox_new(TRUE , 5);
	gtk_widget_show(cpom);
	gtk_container_add (GTK_CONTAINER (frame), cpom);

	rb = gtk_radio_button_new_with_label(NULL , gettext("User auth. scheme"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_container_add (GTK_CONTAINER (cpom), rb);
	gui_cfg.httpp_schgr[0] = rb;

	rb = gtk_radio_button_new_with_label(rg , gettext("Base auth. scheme"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_container_add (GTK_CONTAINER (cpom), rb);
	gui_cfg.httpp_schgr[1] = rb;

	rb = gtk_radio_button_new_with_label(rg , gettext("Digest auth. scheme"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_container_add (GTK_CONTAINER (cpom), rb);
	gui_cfg.httpp_schgr[2] = rb;

	gui_cfg.send_from = gtk_check_button_new_with_label(gettext("Send From: header with HTTP request"));
	gtk_table_attach(GTK_TABLE (col), gui_cfg.send_from , 0 , 2 , 3 , 4 ,
		GTK_FILL , GTK_FILL,  5 , 5);
	gtk_widget_show(gui_cfg.send_from);

	gui_cfg.from_label = guitl_tab_add_entry(col , 
		gettext("E-mail address: ") , 0 , 4 , FALSE);

}

static void cfgtab_ssl(notebook)
GtkWidget *notebook;
{
#ifdef USE_SSL
	GtkWidget *label , *box , *frame , *ptab , *rb, *hbox;
	GSList *rg;

	box = gtk_vbox_new(0,5);
	gtk_widget_show(box);
	label = gtk_label_new(gettext("SSL"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), box , label);

	frame = gtk_frame_new(gettext("SSL client certificate"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(box), frame, FALSE, TRUE, 5);

	ptab = gtk_table_new(4 , 2 , FALSE);
	gtk_widget_show(ptab);
	gtk_container_add(GTK_CONTAINER(frame), ptab);

	gui_cfg.ssl_cert_passwd_en = guitl_tab_add_entry(ptab , 
		gettext("Certificate password: ") , 0 , 0 , TRUE);

	gui_cfg.ssl_cert_file_en = guitl_tab_add_path_entry(ptab , 
		gettext("Certificate PEM file: ") , 0 , 1 , FALSE);

	gui_cfg.ssl_key_file_en = guitl_tab_add_path_entry(ptab , 
		gettext("Certificate key file: ") , 0 , 2 , FALSE);

	gui_cfg.ssl_cipher_list = guitl_tab_add_entry(ptab , 
		gettext("List of preffered ciphers: ") , 0 , 3 , FALSE);

	hbox = gtk_hbox_new(FALSE, 5);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 5);
	gtk_widget_show(hbox);

	frame = gtk_frame_new(gettext("SSL protocol version"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, TRUE, 5);

	ptab = gtk_vbox_new(FALSE, 2);
	gtk_container_add(GTK_CONTAINER(frame), ptab);
	gtk_widget_show(ptab);

	rb = gtk_radio_button_new_with_label(NULL , gettext("SSLv23"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_container_add(GTK_CONTAINER(ptab), rb);
	gui_cfg.ssl_version[0] = rb;

	rb = gtk_radio_button_new_with_label(rg , gettext("SSLv2"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_container_add(GTK_CONTAINER(ptab), rb);
	gui_cfg.ssl_version[1] = rb;

	rb = gtk_radio_button_new_with_label(rg , gettext("SSLv3"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(rb));
	gtk_widget_show(rb);
	gtk_container_add(GTK_CONTAINER(ptab), rb);
	gui_cfg.ssl_version[2] = rb;

	ptab = gtk_vbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(hbox), ptab, FALSE, TRUE, 5);
	gtk_widget_show(ptab);

	gui_cfg.unique_sslid = gtk_check_button_new_with_label(gettext("Unique ID for all SSL sessions"));
	gtk_box_pack_start(GTK_BOX(ptab), gui_cfg.unique_sslid, FALSE, TRUE, 5);
	gtk_widget_show(gui_cfg.unique_sslid);
#endif
}

/*** LOG ***/
static void cfgtab_log(notebook)
GtkWidget *notebook;
{
	GtkWidget *col , *label , *box;

	box = gtk_vbox_new (0,5);
	gtk_widget_show(box);
	label = gtk_label_new (gettext("Log"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), box , label);

	col = gtk_table_new(2 , 4 , FALSE);
	gtk_container_add (GTK_CONTAINER (box), col);
	gtk_widget_show(col);

	gui_cfg.gen_logname = gtk_check_button_new_with_label(gettext("Try to find unique name, when original log file locked"));
	gtk_table_attach(GTK_TABLE(col), gui_cfg.gen_logname, 0 , 2 , 0 , 1 , GTK_FILL , GTK_FILL , 1 , 1);
	gtk_widget_show(gui_cfg.gen_logname);

	gui_cfg.log_label = guitl_tab_add_path_entry(col , 
		gettext("Log file: ") , 0 , 1 , FALSE);

	gui_cfg.slog_label = guitl_tab_add_path_entry(col ,
		gettext("Shortlog file: ") , 0 , 2 , FALSE);

	gui_cfg.xloglen_label = guitl_tab_add_numentry(col , 
		gettext("Log window length: ") , 0 , 3 , INT_MAX);

}

/*** COOKIES ***/
static void cfgtab_cookies(notebook)
GtkWidget *notebook;
{
	GtkWidget *label , *box , *tbox , *hbox, *pom , *frame;
	GtkAdjustment *adj;

	tbox = gtk_hbox_new (0,5);
	gtk_widget_show(tbox);

	hbox = gtk_vbox_new (0,5);
	gtk_widget_show(hbox);
	gtk_box_pack_start(GTK_BOX(tbox) , hbox , TRUE , TRUE , 0);

	gui_cfg.sw_cookie_update = 
		gtk_check_button_new_with_label(gettext("Update cookies"));
	gtk_widget_show(gui_cfg.sw_cookie_update);
	gtk_box_pack_start(GTK_BOX(hbox) , gui_cfg.sw_cookie_update , FALSE , FALSE , 0);

	gui_cfg.sw_cookie_send = 
		gtk_check_button_new_with_label(gettext("Send cookies"));
	gtk_widget_show(gui_cfg.sw_cookie_send);
	gtk_box_pack_start(GTK_BOX(hbox) , gui_cfg.sw_cookie_send , FALSE , FALSE , 0);

	gui_cfg.sw_cookie_recv = 
		gtk_check_button_new_with_label(gettext("Accept cookies"));
	gtk_widget_show(gui_cfg.sw_cookie_recv);
	gtk_box_pack_start(GTK_BOX(hbox) , gui_cfg.sw_cookie_recv , FALSE , FALSE , 0);

	gui_cfg.cookie_check_domain = 
		gtk_check_button_new_with_label(gettext("Check cookies domain"));
	gtk_widget_show(gui_cfg.cookie_check_domain);
	gtk_box_pack_start(GTK_BOX(hbox) , gui_cfg.cookie_check_domain , FALSE , FALSE , 0);

	box = gtk_hbox_new(0,5);
	gtk_widget_show(box);
	gtk_box_pack_start(GTK_BOX(hbox) , box , FALSE , FALSE , 0);

	pom = gtk_label_new(gettext("Cookies maximal number: "));
	gtk_widget_show(pom);
	gtk_box_pack_start(GTK_BOX(box) , pom , FALSE , FALSE , 0);

	adj = (GtkAdjustment *) gtk_adjustment_new(0.0 , 0.0 , 10000.0 , 1.0 , 10.0 , 0.0);
	gui_cfg.en_cookie_max = gtk_spin_button_new(adj , 0 , 0);
	gtk_widget_show(gui_cfg.en_cookie_max);
	gtk_box_pack_start(GTK_BOX(box) , gui_cfg.en_cookie_max , TRUE , FALSE , 0);


	box = gtk_table_new(2, 1, FALSE);
	gtk_widget_show(box);
	gtk_box_pack_start(GTK_BOX(hbox) , box , FALSE , FALSE , 0);

	gui_cfg.en_cookie_file = guitl_tab_add_path_entry(box , 
		gettext("Cookie file: ") , 0 , 0 , FALSE);
/*
	box = gtk_hbox_new(0,5);
	gtk_widget_show(box);
	gtk_box_pack_start(GTK_BOX(hbox) , box , FALSE , FALSE , 0);

	pom = gtk_label_new(gettext("Cookie file: "));
	gtk_widget_show(pom);
	gtk_box_pack_start(GTK_BOX(box) , pom , FALSE , FALSE , 0);

	gui_cfg.en_cookie_file = gtk_entry_new();
	gtk_widget_show(gui_cfg.en_cookie_file);
	gtk_box_pack_start(GTK_BOX(box) , gui_cfg.en_cookie_file , TRUE , TRUE , 0);

	pom = guitl_pixmap_button(browse_xpm , NULL , gettext("Browse ..."));
	gtk_widget_show(pom);
	gtk_box_pack_start(GTK_BOX(box) , pom , FALSE , TRUE , 0);

	gtk_signal_connect(GTK_OBJECT(pom) , "clicked" , 
			GTK_SIGNAL_FUNC(GetFile) , 
			(gpointer)gui_cfg.en_cookie_file);
*/

	frame = gtk_frame_new(gettext("Disabled cookie domains"));
	gtk_widget_show(frame);
	gtk_box_pack_end(GTK_BOX(tbox) , frame , TRUE , TRUE , 5);

	box = guitl_new_edit_list(&gui_cfg.cookie_domain_list , &gui_cfg.cookie_domain_entry ,
                gettext("Domain: ") , NULL , NULL , NULL , NULL , TRUE);
	gtk_container_add (GTK_CONTAINER (frame), box);

	label = gtk_label_new (gettext("Cookies"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), tbox , label);

}

static void RulesSelectRow(w , func_data)
GtkWidget *w;
gpointer func_data;
{
	gchar *p;

#ifdef HAVE_REGEX
	 gtk_clist_get_text(GTK_CLIST(gui_cfg.rules_list) , 
		GPOINTER_TO_INT(GTK_CLIST(gui_cfg.rules_list)->selection->data),
		0 , &p);

	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(gui_cfg.ptrn_fnmatch) , (*p == 'F'));
#endif

	 gtk_clist_get_text(GTK_CLIST(gui_cfg.rules_list) , 
		GPOINTER_TO_INT(GTK_CLIST(gui_cfg.rules_list)->selection->data),
		1 , &p);
	gtk_entry_set_text(GTK_ENTRY(gui_cfg.mpt_entry) , p);
	
	 gtk_clist_get_text(GTK_CLIST(gui_cfg.rules_list) , 
		GPOINTER_TO_INT(GTK_CLIST(gui_cfg.rules_list)->selection->data),
		2 , &p);
	gtk_entry_set_text(GTK_ENTRY(gui_cfg.rule_entry) , p);
}

static void RulesAppend(w , func_data)
GtkWidget *w;
gpointer func_data;
{
	gchar *p[3];

	p[1] = gtk_entry_get_text(GTK_ENTRY(gui_cfg.mpt_entry));
	p[2] = gtk_entry_get_text(GTK_ENTRY(gui_cfg.rule_entry));

#ifdef HAVE_REGEX
	p[0] = GTK_TOGGLE_BUTTON(gui_cfg.ptrn_fnmatch)->active ? "F" : "R";

	if (!lfname_check_pattern(GTK_TOGGLE_BUTTON(gui_cfg.ptrn_fnmatch)->active ?
		LFNAME_FNMATCH : LFNAME_REGEX , p[1]))
	{
		gdk_beep();
		return;
	}
#else
	p[0] = "F";
#endif


	if (p[1] && p[2] && *p[1] && *p[2] && lfname_check_rule(p[2]))
		gtk_clist_append(GTK_CLIST(gui_cfg.rules_list) , p);
	else
		gdk_beep();
}

static void RulesModify(w , func_data)
GtkWidget *w;
gpointer func_data;
{
	gchar *p[3];
	int row;

        if (!GTK_CLIST(gui_cfg.rules_list)->selection)
        {
                gdk_beep();
                return;
        }

	row = GPOINTER_TO_INT(GTK_CLIST(gui_cfg.rules_list)->selection->data);

	p[1] = gtk_entry_get_text(GTK_ENTRY(gui_cfg.mpt_entry));
	p[2] = gtk_entry_get_text(GTK_ENTRY(gui_cfg.rule_entry));

#ifdef HAVE_REGEX
	p[0] = GTK_TOGGLE_BUTTON(gui_cfg.ptrn_fnmatch)->active ? "F" : "R";

	if (!lfname_check_pattern(GTK_TOGGLE_BUTTON(gui_cfg.ptrn_fnmatch)->active ?
		LFNAME_FNMATCH : LFNAME_REGEX , p[1]))
	{
		gdk_beep();
		return;
	}
#else
	p[0] = "F";
#endif

	if (p[1] && p[2] && *p[1] && *p[2])
	{
		gtk_clist_set_text(GTK_CLIST(gui_cfg.rules_list) , row , 0 , p[0]);
		gtk_clist_set_text(GTK_CLIST(gui_cfg.rules_list) , row , 1 , p[1]);
		gtk_clist_set_text(GTK_CLIST(gui_cfg.rules_list) , row , 2 , p[2]);
	}
	else
		gdk_beep();
}

static void cfgtab_filename(notebook)
GtkWidget *notebook;
{
	GtkWidget *label , *box , *frame , *ptab , *hbox , *swin , *button , *bbox;

	hbox = gtk_hbox_new (FALSE,5);
	gtk_widget_show(hbox);
	label = gtk_label_new (gettext("Filename"));
	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), hbox , label);

	frame = gtk_frame_new(gettext("Local filename conversion rules"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(hbox) , frame , TRUE , FALSE , 5);

	box = gtk_vbox_new (FALSE,5);
	gtk_widget_show(box);
	gtk_container_add (GTK_CONTAINER (frame), box);

	frame = gtk_frame_new(gettext("Replace String1 with String2 in filename"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(box) , frame , FALSE , FALSE , 5);

	ptab = gtk_table_new(2 , 2 , FALSE);
	gtk_container_add (GTK_CONTAINER (frame), ptab);
	gtk_widget_show(ptab);

	gui_cfg.tr_str_s1 = guitl_tab_add_entry(ptab , 
		gettext("String1: ") , 0 , 0 , FALSE);

	gui_cfg.tr_str_s2 = guitl_tab_add_entry(ptab , 
		gettext("String2: ") , 0 , 1 , FALSE);

	frame = gtk_frame_new(gettext("Delete characters from filename"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(box) , frame , FALSE , FALSE , 5);

	ptab = gtk_table_new(2 , 1 , FALSE);
	gtk_container_add (GTK_CONTAINER (frame), ptab);
	gtk_widget_show(ptab);

	gui_cfg.tr_del_chr = guitl_tab_add_entry(ptab , 
		gettext("Character set: ") , 0 , 0 , FALSE);

	frame = gtk_frame_new(gettext("Replace chars from Set1 with chars from Set2 in filename"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(box) , frame , FALSE , FALSE , 5);

	ptab = gtk_table_new(2 , 2 , FALSE);
	gtk_container_add (GTK_CONTAINER (frame), ptab);
	gtk_widget_show(ptab);

	gui_cfg.tr_chr_s1 = guitl_tab_add_entry(ptab , 
		gettext("Character set1: ") , 0 , 0 , FALSE);

	gui_cfg.tr_chr_s2 = guitl_tab_add_entry(ptab , 
		gettext("Character set2: ") , 0 , 1 , FALSE);

	ptab = gtk_table_new(2 , 1 , FALSE);
	gtk_box_pack_start(GTK_BOX(box) , ptab , FALSE, FALSE , 5);
	gtk_widget_show(ptab);

	gui_cfg.base_level_label = guitl_tab_add_numentry(ptab , 
		gettext("Base level of tree: ") , 0 , 0 , USHRT_MAX);

	frame = gtk_frame_new(gettext("Local filename mapping rules"));
	gtk_widget_show(frame);
	gtk_box_pack_start(GTK_BOX(hbox) , frame , TRUE , TRUE , 5);
	
	box = gtk_vbox_new(0,5);
	gtk_widget_show(box);
	gtk_container_add (GTK_CONTAINER (frame), box);

	swin = gtk_scrolled_window_new(NULL , NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (swin),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(box) , swin , TRUE , TRUE , 5);
	gtk_widget_show (swin);

	gui_cfg.rules_list = gtk_clist_new(3);
	gtk_clist_set_column_title(GTK_CLIST(gui_cfg.rules_list) , 0 , gettext("Type"));
	gtk_clist_set_column_title(GTK_CLIST(gui_cfg.rules_list) , 1 , gettext("Match pattern"));
	gtk_clist_set_column_title(GTK_CLIST(gui_cfg.rules_list) , 2 , gettext("Rule"));
	gtk_clist_column_titles_show(GTK_CLIST(gui_cfg.rules_list));
#ifdef _GTK_FEATURES_1_2
	gtk_clist_set_reorderable(GTK_CLIST(gui_cfg.rules_list) , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(gui_cfg.rules_list) , 0 , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(gui_cfg.rules_list) , 1 , TRUE);
	gtk_clist_set_column_auto_resize(GTK_CLIST(gui_cfg.rules_list) , 2 , TRUE);
#endif
	gtk_signal_connect(GTK_OBJECT(gui_cfg.rules_list) , "select_row" ,
		GTK_SIGNAL_FUNC(RulesSelectRow) , NULL);
	gtk_container_add(GTK_CONTAINER(swin), gui_cfg.rules_list);
	gtk_widget_show(gui_cfg.rules_list);

#ifdef HAVE_REGEX
{
	GSList *rg = NULL;

	ptab = gtk_table_new(2 , 1 , FALSE);
	gtk_box_pack_start(GTK_BOX(box) , ptab , FALSE , TRUE , 2);
	gtk_widget_show(ptab);

	gui_cfg.ptrn_fnmatch = 
		gtk_radio_button_new_with_label(NULL , gettext("wildcard pattern"));
	rg = gtk_radio_button_group(GTK_RADIO_BUTTON(gui_cfg.ptrn_fnmatch));
	gtk_widget_show(gui_cfg.ptrn_fnmatch);
	gtk_table_attach(GTK_TABLE(ptab) , gui_cfg.ptrn_fnmatch , 0 , 1 , 0 , 1 ,
		GTK_FILL , GTK_FILL , 2 , 2);

	gui_cfg.ptrn_regex = 
		gtk_radio_button_new_with_label(rg , gettext("RE pattern"));
	gtk_widget_show(gui_cfg.ptrn_regex);
	gtk_table_attach(GTK_TABLE(ptab) , gui_cfg.ptrn_regex , 1 , 2 , 0 , 1 ,
		GTK_FILL , GTK_FILL , 2 , 2);
}
#endif

	ptab = gtk_table_new(2 , 1 , FALSE);
	gtk_box_pack_start(GTK_BOX(box) , ptab , FALSE , TRUE , 2);
	gtk_widget_show(ptab);

	gui_cfg.mpt_entry = guitl_tab_add_entry(ptab , 
		gettext("Matching pattern: ") , 0 , 0 , FALSE);

	ptab = gtk_table_new(2 , 1 , FALSE);
	gtk_box_pack_start(GTK_BOX(box) , ptab , FALSE , TRUE , 2);
	gtk_widget_show(ptab);

	gui_cfg.rule_entry = guitl_tab_add_entry(ptab , 
		gettext("Construction rule: ") , 0 , 0 , FALSE);

	bbox = gtk_hbutton_box_new();
	gtk_widget_show(bbox);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox) , GTK_BUTTONBOX_SPREAD);
	gtk_box_pack_start(GTK_BOX(box) , bbox , FALSE , TRUE , 5);

	button = guitl_pixmap_button(append_xpm , NULL , gettext("Append"));
	gtk_container_add(GTK_CONTAINER(bbox) , button);
	gtk_widget_show(button);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(RulesAppend), NULL);

	button = guitl_pixmap_button(modify_xpm , NULL , gettext("Modify"));
	gtk_container_add(GTK_CONTAINER(bbox) , button);
	gtk_widget_show(button);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(RulesModify), NULL);

	button = guitl_pixmap_button(clear_xpm , NULL , gettext("Clear"));
	gtk_container_add(GTK_CONTAINER(bbox) , button);
	gtk_widget_show(button);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(guitl_ListClear), gui_cfg.rules_list);

	button = guitl_pixmap_button(delete_xpm , NULL , gettext("Delete"));
	gtk_container_add(GTK_CONTAINER(bbox) , button);
	gtk_widget_show(button);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(guitl_ListDeleteSelected), gui_cfg.rules_list);
}

static void cfgtab_advert(notebook)
GtkWidget *notebook;
{
	GtkWidget *hbox, *vbox, *label, *box, *entry;

	hbox = gtk_hbox_new(FALSE,5);
	gtk_widget_show(hbox);
	label = gtk_label_new(gettext("Advertisement"));
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox , label);

	vbox = gtk_vbox_new(FALSE ,1);
	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE , 1);
	gtk_widget_show(vbox);

	gui_cfg.remove_adv = gtk_check_button_new_with_label(gettext("Enable removing of advertisement banners"));
	gtk_box_pack_start(GTK_BOX(vbox), gui_cfg.remove_adv, FALSE, FALSE , 1);
	gtk_widget_show(gui_cfg.remove_adv);

	box = guitl_new_edit_list(&gui_cfg.advert_res , &entry ,
                gettext("RE for advertisement URLs: ") , NULL , NULL , NULL , NULL , TRUE);
	gtk_box_pack_start(GTK_BOX(vbox), box, TRUE, TRUE , 1);

}

/*** COMMON CFG ***/
void gui_build_config_common(popup)
int popup;
{
	GtkWidget *col , *brow , *button , *notebook;

	if (gui_cfg.config_shell)
	{
		if (popup)
		{
		 	gtk_widget_show_all(gui_cfg.config_shell);
			if (GTK_WIDGET_REALIZED(gui_cfg.config_shell))
				gdk_window_raise(gui_cfg.config_shell->window);
		}
		return;
	}

	gui_cfg.config_shell = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_container_border_width(GTK_CONTAINER(gui_cfg.config_shell), 3);
	gtk_window_set_title(GTK_WINDOW(gui_cfg.config_shell) , gettext("Pavuk: Common config"));
	gtk_widget_realize(gui_cfg.config_shell);
	gtk_signal_connect(GTK_OBJECT(gui_cfg.config_shell), "destroy",
		GTK_SIGNAL_FUNC(gtk_widget_destroyed), &gui_cfg.config_shell);

	col = gtk_table_new(2 , 1 , FALSE);
	gtk_container_add(GTK_CONTAINER(gui_cfg.config_shell), col);
	gtk_widget_show(col);

	gui_cfg.cb_comcfg = notebook = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook) , TRUE);
	gtk_table_attach_defaults(GTK_TABLE(col), notebook , 0 , 1, 0, 1);
	gtk_widget_show(notebook);

	cfgtab_url(notebook);

	cfgtab_formdata(notebook);

	cfgtab_graberI(notebook);

	cfgtab_graberII(notebook);
	
	cfgtab_net(notebook);

	cfgtab_proxy(notebook);

	cfgtab_lang(notebook);

	cfgtab_charset(notebook);

	cfgtab_auth(notebook);

	cfgtab_ssl(notebook);

	cfgtab_log(notebook);

	cfgtab_cookies(notebook);

	cfgtab_filename(notebook);

	cfgtab_advert(notebook);

	brow = gtk_hbutton_box_new();
	gtk_table_attach(GTK_TABLE(col), brow , 0, 1, 1, 2,
			GTK_EXPAND | GTK_FILL, GTK_FILL, 2 , 5);
	gtk_hbutton_box_set_spacing_default(1);
	gtk_widget_show(brow);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(brow) , GTK_BUTTONBOX_SPREAD);

	button = guitl_pixmap_button(ok_xpm , NULL , gettext("OK"));
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
			GTK_SIGNAL_FUNC(gui_PopdownWC) , 
			(gpointer)gui_cfg.config_shell);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(button);
	gtk_widget_show(button);

	button = guitl_pixmap_button(apply_xpm , NULL , gettext("Apply"));
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
			GTK_SIGNAL_FUNC(CfgCommon) , 
			(gpointer)gui_cfg.config_shell);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_show(button);

	button = guitl_pixmap_button(limit_xpm , NULL , gettext("Limitations ..."));
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
			GTK_SIGNAL_FUNC(gui_PopupW) , 
			(gpointer)PAVUK_CFGLIM);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_show(button);

	button = guitl_pixmap_button(cancel_xpm , NULL , gettext("Cancel"));
#ifdef _GTK_FEATURES_1_2
{
	GtkAccelGroup *accel_group;
	accel_group = gtk_accel_group_new();

	gtk_widget_add_accelerator(button , "clicked" , accel_group ,
			GDK_Escape , 0 , GTK_ACCEL_VISIBLE);

	gtk_window_add_accel_group(GTK_WINDOW(gui_cfg.config_shell) , accel_group);
}
#endif
	gtk_container_add(GTK_CONTAINER(brow), button);
	gtk_signal_connect(GTK_OBJECT(button) , "clicked" , 
			GTK_SIGNAL_FUNC(guitl_PopdownW) , 
			(gpointer)gui_cfg.config_shell);
	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
	gtk_widget_show(button);

	if (popup) gtk_widget_show(gui_cfg.config_shell);

	xset_cfg_values_comm();
}

#endif /* GTK_FACE */
