/***************************************************************************/
/* 		This code is part of WWW grabber called pavuk		   */
/*		Copyright (c) 1997 - 2000 Stefan Ondrejicka		   */
/*		(ondrej@idata.sk)					   */
/*		Distributed under GPL 2 or later			   */
/***************************************************************************/

#ifdef HAVE_MT
#include <stdio.h>
#include "config.h"
#include "gui.h"
#include "mt.h"

pthread_mutex_t _mt_urlstack_lock;
pthread_mutex_t _mt_urlhash_lock;
pthread_mutex_t _mt_filehash_lock;
pthread_mutex_t _mt_cookies_lock;
pthread_mutex_t _mt_authinfo_lock;
pthread_mutex_t _mt_getlfname_lock;
pthread_mutex_t _mt_dns_lock;
pthread_mutex_t _mt_log_lock;
pthread_mutex_t _mt_slog_lock;
pthread_mutex_t _mt_dbase_lock;
pthread_mutex_t _mt_dcnt_lock;
pthread_mutex_t _mt_time_lock;
pthread_mutex_t _mt_ghbn_lock;
pthread_mutex_t _mt_output_lock;
pthread_mutex_t _mt_proxy_lock;
pthread_mutex_t _mt_dirr_lock;
pthread_mutex_t _mt_tcnt_lock;
pthread_mutex_t _mt_gtktree_lock;
pthread_mutex_t _mt_gtkstatus_lock;
pthread_mutex_t _mt_gtklog_lock;
pthread_mutex_t _mt_rejcnt_lock;
pthread_mutex_t _mt_failcnt_lock;
pthread_mutex_t _mt_gcfg_lock;
pthread_mutex_t _mt_nscache_lock;

void mt_init()
{
        static int __h_errno;

	pthread_mutex_init(&_mt_urlstack_lock, NULL);
	pthread_mutex_init(&_mt_urlhash_lock, NULL);
	pthread_mutex_init(&_mt_filehash_lock, NULL);
	pthread_mutex_init(&_mt_cookies_lock, NULL);
	pthread_mutex_init(&_mt_authinfo_lock, NULL);
	pthread_mutex_init(&_mt_dns_lock, NULL);
	pthread_mutex_init(&_mt_log_lock, NULL);
	pthread_mutex_init(&_mt_slog_lock, NULL);
	pthread_mutex_init(&_mt_dcnt_lock, NULL);
	pthread_mutex_init(&_mt_time_lock, NULL);
	pthread_mutex_init(&_mt_ghbn_lock, NULL);
	pthread_mutex_init(&_mt_getlfname_lock, NULL);
	pthread_mutex_init(&_mt_output_lock, NULL);
	pthread_mutex_init(&_mt_proxy_lock, NULL);
	pthread_mutex_init(&_mt_dirr_lock, NULL);
	pthread_mutex_init(&_mt_tcnt_lock, NULL);
	pthread_mutex_init(&_mt_gtktree_lock, NULL);
	pthread_mutex_init(&_mt_gtkstatus_lock, NULL);
	pthread_mutex_init(&_mt_gtklog_lock, NULL);
	pthread_mutex_init(&_mt_rejcnt_lock, NULL);
	pthread_mutex_init(&_mt_failcnt_lock, NULL);
	pthread_mutex_init(&_mt_gcfg_lock, NULL);
	pthread_mutex_init(&_mt_nscache_lock, NULL);

	pthread_key_create(&cfg.currdoc_key, NULL);
	pthread_key_create(&cfg.herrno_key, NULL);
	pthread_key_create(&cfg.thrnr_key, NULL);
	pthread_key_create(&cfg.privcfg_key, NULL);

	mt_semaphore_init(&cfg.nrunning_sem);
	mt_semaphore_init(&cfg.urlstack_sem);

	pthread_setspecific(cfg.herrno_key, &__h_errno);
	pthread_setspecific(cfg.currdoc_key, NULL);
	pthread_setspecific(cfg.thrnr_key, (void *)-1);
	pthread_setspecific(cfg.privcfg_key, &cfg);

	cfg.cfg_changed = 0L;

#ifdef _GTK_FEATURES_1_2
	g_thread_init(NULL);
#endif
}

int mt_pthread_mutex_lock(mutex, id)
pthread_mutex_t *mutex;
char *id;
{
	int rv;

	DEBUG_MTLOCK("Try lock(%ld) - %s\n", pthread_self(), id);
	rv = pthread_mutex_lock(mutex);
	DEBUG_MTLOCK("Locking(%ld) - %s\n", pthread_self(), id);

	return rv;
}

int mt_pthread_mutex_unlock(mutex, id)
pthread_mutex_t *mutex;
char *id;
{
	DEBUG_MTLOCK("Unlocking(%ld) - %s\n", pthread_self(), id);
	return pthread_mutex_unlock(mutex);
}

/********************************************************/
/* semaphore code based on Tom Wagner and Don Towsley	*/
/* document "Getting Started With POSIX Threads"	*/
/********************************************************/
int mt_semaphore_init(sem)
mt_semaphore *sem;
{
	sem->v = 1;
	if (pthread_mutex_init(&(sem->mutex), NULL))
	{
		xperror("mt_semaphore_init - mutex_init");
		return -1;
	}
	if (pthread_cond_init(&(sem->cond), NULL))
	{
		xperror("mt_semaphore_init - cond_init");
		return -1;
	}
	return 0;
}

int mt_semaphore_destroy(sem)
mt_semaphore *sem;
{
	if (pthread_mutex_destroy(&(sem->mutex)))
	{
		xperror("mt_semaphore_destroy - mutex_destroy");
		return -1;
	}
	if (pthread_cond_destroy(&(sem->cond)))
	{
		xperror("mt_semaphore_destroy - cond_destroy");
		return -1;
	}
	return 0;
}

long mt_semaphore_up(sem)
mt_semaphore *sem;
{
	long rv;

	pthread_mutex_lock(&(sem->mutex));

	sem->v ++;
	rv = sem->v;

	pthread_mutex_unlock(&(sem->mutex));
	pthread_cond_signal(&(sem->cond));

	return rv;
}

long mt_semaphore_down(sem)
mt_semaphore *sem;
{
	long rv;

	pthread_mutex_lock(&(sem->mutex));

	while(sem->v <= 0)
	{
		pthread_cond_wait(&(sem->cond), &(sem->mutex));
	}

	sem->v --;
	rv = sem->v;

	pthread_mutex_unlock(&(sem->mutex));

	return rv;
}

long mt_semaphore_decrement(sem)
mt_semaphore *sem;
{
	long rv;

	pthread_mutex_lock(&(sem->mutex));

	sem->v --;
	rv = sem->v;

	pthread_mutex_unlock(&(sem->mutex));

	return rv;
}

#endif

